#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
from subprocess import PIPE, run, STDOUT

import reusables

from qtpy import QtWidgets, QtCore, QtGui

logger = logging.getLogger("fastflix")

__all__ = ["ThumbnailCreator"]


class ThumbnailCreator(QtCore.QThread):
    def __init__(self, app, command=""):
        super().__init__(app)
        self.app = app
        self.command = command

    def run(self):
        logger.debug(f"Generating thumbnail: {self.command}")
        result = run(self.command, stdin=PIPE, stdout=PIPE, stderr=STDOUT, shell=True)
        if result.returncode > 0:
            logger.error(f"Could not generate thumbnail: {result.stdout}")
        else:
            self.app.thumbnail_complete.emit()



# -*- coding: utf-8 -*-


class Loop:

    item = "loop"

    def __init__(self, condition, commands, dirs=(), files=(), name="", ensure_paths=()):
        self.name = name
        self.condition = condition
        self.commands = commands
        self.ensure_paths = ensure_paths
        self.dirs = dirs
        self.files = files


class Command:

    item = "command"

    def __init__(self, command, variables, internal, name="", ensure_paths=(), exe=None):
        self.name = name
        self.command = command
        self.variables = variables
        self.internal = internal
        self.ensure_paths = ensure_paths
        self.exe = exe


def start_and_input(source, ffmpeg, **kwargs):
    start_time = kwargs.get("start_time", 0) or 0
    duration = kwargs.get("duration")

    return (
        f'"{ffmpeg}" -i "{source}" -y {f"-ss {start_time}" if start_time else ""} '
        f'{f"-t {duration - start_time}" if duration else ""} '
    )


def generate_filters(**kwargs):
    crop = kwargs.get("crop")
    scale = kwargs.get("scale")
    scale_filter = kwargs.get("scale_filter", "lanczos")
    scale_width = kwargs.get("scale_width")
    scale_height = kwargs.get("scale_height")
    disable_hdr = kwargs.get("disable_hdr")
    rotate = kwargs.get("rotate")
    vflip = kwargs.get("v_flip")
    hflip = kwargs.get("h_flip")

    filter_list = []
    if crop:
        filter_list.append(f"crop={crop}")
    if scale:
        filter_list.append(f"scale={scale}:flags={scale_filter}")
    elif scale_width:
        filter_list.append(f"scale={scale_width}:-1:flags={scale_filter}")
    elif scale_height:
        filter_list.append(f"scale=-1:{scale_height}:flags={scale_filter}")
    if rotate is not None:
        if rotate <= 3:
            filter_list.append(f"transpose={rotate}")
        if rotate == 4:
            filter_list.append(f"transpose=2,transpose=2")
    if vflip:
        filter_list.append("vflip")
    if hflip:
        filter_list.append("hflip")

    if disable_hdr:
        filter_list.append(
            "zscale=t=linear:npl=100,format=gbrpf32le,zscale=p=bt709,tonemap=tonemap=hable:desat=0,"
            "zscale=t=bt709:m=bt709:r=tv,format=yuv420p"
        )

    return ",".join(filter_list)


# def load_external_plugins(plugin_dir, configuration):
#     sys.path.insert(0, str(Path(plugin_dir, os.pardir)))
#     plugins = Box()
#     for item in plugin_dir.iterdir():
#         if item.is_dir() and not item.name.startswith("_") and item.name != "common":
#             plugin = importlib.machinery.SourceFileLoader(
#                 f"plugin_{item.name}", str(Path(item, "main.py"))
#             ).load_module()
#             requires = getattr(plugin, "requires", None)
#             if not requires or (requires and requires in configuration):
#                 plugins[plugin.name] = plugin
#     return plugins



# -*- coding: utf-8 -*-
import logging

from box import Box

from qtpy import QtWidgets, QtCore, QtGui

logger = logging.getLogger("fastflix")

recommended_bitrates = [
    "150k   (320x240p @ 24,25,30)",
    "276k   (640x360p @ 24,25,30)",
    "512k   (640x480p @ 24,25,30)",
    "1024k  (1280x720p @ 24,25,30)",
    "1800k (1280x720p @ 50,60)",
    "1800k (1920x1080p @ 24,25,30)",
    "3000k (1920x1080p @ 50,60)",
    "6000k (2560x1440p @ 24,25,30)",
    "9000k (2560x1440p @ 50,60)",
    "12000k (3840x2160p @ 24,25,30)",
    "18000k (3840x2160p @ 50,60)",
    "Custom",
]

recommended_crfs = [
    "28",
    "27",
    "26",
    "25",
    "24",
    "23",
    "22",
    "21",
    "20",
    "19",
    "18",
    "Custom",
]

pix_fmts = ["8-bit: yuv420p", "10-bit: yuv420p10le", "12-bit: yuv420p12le"]


class HEVC(QtWidgets.QWidget):
    def __init__(self, parent, main):
        super(HEVC, self).__init__(parent)
        self.main = main

        grid = QtWidgets.QGridLayout()

        self.widgets = Box(remove_hdr=None, mode=None, intra_encoding=None)

        self.mode = "CRF"
        self.updating_settings = False

        grid.addLayout(self.init_modes(), 0, 2, 6, 4)
        grid.addLayout(self.init_custom(), 6, 2, 8, 4)

        grid.addLayout(self.init_preset(), 1, 0, 1, 2)
        grid.addLayout(self.init_remove_hdr(), 2, 0, 1, 2)
        grid.addLayout(self.init_intra_encoding(), 3, 0, 1, 2)
        grid.addLayout(self.init_max_mux(), 4, 0, 1, 2)
        grid.addLayout(self.init_tune(), 5, 0, 1, 2)
        grid.addLayout(self.init_pix_fmt(), 6, 0, 1, 2)
        grid.addLayout(self.init_profile(), 7, 0, 1, 2)

        grid.addWidget(QtWidgets.QWidget(), 8, 0)
        grid.setRowStretch(8, 1)

        guide_label = QtWidgets.QLabel(
            "<a href='https://trac.ffmpeg.org/wiki/Encode/H.265'>FFMPEG HEVC / H.265 Encoding Guide</a>"
            " | <a href='https://codecalamity.com/encoding-uhd-4k-hdr10-videos-with-ffmpeg/'>"
            "CodeCalamity UHD HDR Encoding Guide</a>"
        )
        guide_label.setAlignment(QtCore.Qt.AlignBottom)
        guide_label.setOpenExternalLinks(True)
        grid.addWidget(guide_label, 9, 0, -1, 1)

        self.setLayout(grid)
        self.hide()

    def init_remove_hdr(self):
        layout = QtWidgets.QHBoxLayout()
        self.remove_hdr_label = QtWidgets.QLabel("Remove HDR")
        self.remove_hdr_label.setToolTip(
            "Convert BT2020 colorspace into bt709\n " "WARNING: This will take much longer and result in a larger file"
        )
        layout.addWidget(self.remove_hdr_label)
        self.widgets.remove_hdr = QtWidgets.QComboBox()
        self.widgets.remove_hdr.addItems(["No", "Yes"])
        self.widgets.remove_hdr.setCurrentIndex(0)
        self.widgets.remove_hdr.setDisabled(True)
        self.widgets.remove_hdr.currentIndexChanged.connect(lambda: self.setting_change())
        layout.addWidget(self.widgets.remove_hdr)
        return layout

    def init_intra_encoding(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Intra-encoding")
        label.setToolTip(
            "Enable Intra-Encoding by forcing keyframes every 1 second (Blu-ray spec)\n"
            "This option is not recommenced unless you need to conform to Blu-ray standards to burn to a physical disk"
        )
        layout.addWidget(label)
        self.widgets.intra_encoding = QtWidgets.QComboBox()
        self.widgets.intra_encoding.addItems(["No", "Yes"])
        self.widgets.intra_encoding.setCurrentIndex(0)
        self.widgets.intra_encoding.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.intra_encoding)
        return layout

    def init_preset(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Preset")
        label.setToolTip(
            "The slower the preset, the better the compression and quality\n"
            "Slow is highest personal recommenced, as past that is much smaller gains"
        )
        layout.addWidget(label)
        self.widgets.preset = QtWidgets.QComboBox()
        self.widgets.preset.addItems(
            ["ultrafast", "superfast", "veryfast", "faster", "fast", "medium", "slow", "slower", "veryslow", "placebo"]
        )
        self.widgets.preset.setCurrentIndex(5)
        self.widgets.preset.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.preset)
        return layout

    def init_tune(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Tune")
        label.setToolTip("Tune the settings for a particular type of source or situation")
        layout.addWidget(label)
        self.widgets.tune = QtWidgets.QComboBox()
        self.widgets.tune.addItems(["default", "psnr", "ssim", "grain", "zero-latency", "fast-decode", "animation"])
        self.widgets.tune.setCurrentIndex(0)
        self.widgets.tune.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.tune)
        return layout

    def init_profile(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Profile")
        label.setToolTip("Enforce an encode profile")
        layout.addWidget(label)
        self.widgets.profile = QtWidgets.QComboBox()
        self.widgets.profile.addItems(["default", "main", "main10", "mainstillpicture"])
        self.widgets.profile.setCurrentIndex(0)
        self.widgets.profile.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.profile)
        return layout

    def init_pix_fmt(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Bit Depth")
        label.setToolTip("Pixel Format (requires at least 10-bit for HDR)")
        layout.addWidget(label)
        self.widgets.pix_fmt = QtWidgets.QComboBox()
        self.widgets.pix_fmt.addItems(pix_fmts)
        self.widgets.pix_fmt.setCurrentIndex(1)
        self.widgets.pix_fmt.currentIndexChanged.connect(lambda: self.setting_change(pix_change=True))
        layout.addWidget(self.widgets.pix_fmt)
        return layout

    def init_max_mux(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Max Muxing Queue Size")
        label.setToolTip('Only change this if you are getting the error "Too many packets buffered for output stream"')
        layout.addWidget(label)
        self.widgets.max_mux = QtWidgets.QComboBox()
        self.widgets.max_mux.addItems(["default", "1024", "2048", "4096", "8192"])
        self.widgets.max_mux.setCurrentIndex(0)
        self.widgets.max_mux.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.max_mux)
        return layout

    def init_custom(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Custom ffmpeg options")
        label.setToolTip("extra flags or options, cannot modify existing settings")
        layout.addWidget(label)
        self.widgets.extra = QtWidgets.QLineEdit()
        self.widgets.extra.textChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.extra)
        return layout

    def init_modes(self):
        layout = QtWidgets.QGridLayout()
        crf_group_box = QtWidgets.QGroupBox()
        crf_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        crf_box_layout = QtWidgets.QHBoxLayout()
        bitrate_group_box = QtWidgets.QGroupBox()
        bitrate_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        bitrate_box_layout = QtWidgets.QHBoxLayout()
        self.widgets.mode = QtWidgets.QButtonGroup()
        self.widgets.mode.buttonClicked.connect(self.set_mode)

        bitrate_radio = QtWidgets.QRadioButton("Bitrate")
        bitrate_radio.setFixedWidth(80)
        self.widgets.mode.addButton(bitrate_radio)
        self.widgets.bitrate = QtWidgets.QComboBox()
        self.widgets.bitrate.setFixedWidth(250)
        self.widgets.bitrate.addItems(recommended_bitrates)
        self.widgets.bitrate.setCurrentIndex(6)
        self.widgets.bitrate.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_bitrate = QtWidgets.QLineEdit("3000")
        self.widgets.custom_bitrate.setFixedWidth(100)
        self.widgets.custom_bitrate.setDisabled(True)
        self.widgets.custom_bitrate.textChanged.connect(lambda: self.main.build_commands())
        bitrate_box_layout.addWidget(bitrate_radio)
        bitrate_box_layout.addWidget(self.widgets.bitrate)
        bitrate_box_layout.addStretch()
        bitrate_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        bitrate_box_layout.addWidget(self.widgets.custom_bitrate)

        crf_radio = QtWidgets.QRadioButton("CRF")
        crf_radio.setChecked(True)
        crf_radio.setFixedWidth(80)
        crf_radio.setToolTip("28 is x265's default<br>"
                             "24 is  \"Visually Indistinguishable\"<br>"
                             "22 is near what I use for 4K Videos")
        self.widgets.mode.addButton(crf_radio)

        self.widgets.crf = QtWidgets.QComboBox()
        self.widgets.crf.setFixedWidth(250)
        self.widgets.crf.addItems(recommended_crfs)
        self.widgets.crf.setCurrentIndex(0)
        self.widgets.crf.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_crf = QtWidgets.QLineEdit("30")
        self.widgets.custom_crf.setFixedWidth(100)
        self.widgets.custom_crf.setDisabled(True)
        self.widgets.custom_crf.textChanged.connect(lambda: self.main.build_commands())
        crf_box_layout.addWidget(crf_radio)
        crf_box_layout.addWidget(self.widgets.crf)
        crf_box_layout.addStretch()
        crf_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        crf_box_layout.addWidget(self.widgets.custom_crf)

        bitrate_group_box.setLayout(bitrate_box_layout)
        crf_group_box.setLayout(crf_box_layout)

        layout.addWidget(crf_group_box, 0, 0)
        layout.addWidget(bitrate_group_box, 1, 0)
        return layout

    def mode_update(self):
        self.widgets.custom_crf.setDisabled(self.widgets.crf.currentText() != "Custom")
        self.widgets.custom_bitrate.setDisabled(self.widgets.bitrate.currentText() != "Custom")
        self.main.build_commands()

    def setting_change(self, update=True, pix_change=False):
        if self.updating_settings:
            return
        if pix_change:
            self.main.page_update()
            return
        self.updating_settings = True
        remove_hdr = bool(self.widgets.remove_hdr.currentIndex())
        bit_depth = self.main.streams["video"][self.main.video_track].bit_depth

        if remove_hdr:
            self.widgets.pix_fmt.clear()
            self.widgets.pix_fmt.addItems([pix_fmts[0:1]])
            self.widgets.pix_fmt.setCurrentIndex(0)
        else:
            self.widgets.pix_fmt.clear()
            if bit_depth == 12:
                self.widgets.pix_fmt.addItems(pix_fmts[2:])
                self.widgets.pix_fmt.setCurrentIndex(0)
            elif bit_depth == 10:
                self.widgets.pix_fmt.addItems(pix_fmts[1:])
                self.widgets.pix_fmt.setCurrentIndex(0)
            else:
                self.widgets.pix_fmt.addItems(pix_fmts)
                self.widgets.pix_fmt.setCurrentIndex(1)
        if update:
            self.main.page_update()
        self.updating_settings = False

    def get_settings(self):
        settings = Box(
            disable_hdr=bool(self.widgets.remove_hdr.currentIndex()),
            preset=self.widgets.preset.currentText(),
            intra_encoding=bool(self.widgets.intra_encoding.currentIndex()),
            max_mux=self.widgets.max_mux.currentText(),
            extra=self.widgets.extra.text(),
            pix_fmt=self.widgets.pix_fmt.currentText().split(":")[1].strip(),
            profile=self.widgets.profile.currentText(),
        )

        tune = self.widgets.tune.currentText()
        settings.tune = tune if tune.lower() != "default" else None

        if self.mode == "CRF":
            crf = self.widgets.crf.currentText()
            settings.crf = int(crf.split(" ", 1)[0]) if crf != "Custom" else int(self.widgets.custom_crf.text())
        else:
            bitrate = self.widgets.bitrate.currentText()
            if bitrate.lower() == "custom":
                settings.bitrate = self.widgets.custom_bitrate.currentText()
            else:
                settings.bitrate = bitrate.split(" ", 1)[0]
        return settings

    def new_source(self):
        if not self.main.streams:
            return
        if "zcale" not in self.main.flix.filters:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#777}")
            self.remove_hdr_label.setToolTip("cannot remove HDR, zcale filter not in current version of FFmpeg")
            logger.warning("zcale filter not detected in current version of FFmpeg, cannot remove HDR")
        elif self.main.streams["video"][self.main.video_track].get("color_space", "").startswith("bt2020"):
            self.widgets.remove_hdr.setDisabled(False)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")
        else:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")
        self.setting_change(update=False)

    def set_mode(self, x):
        self.mode = x.text()
        self.main.build_commands()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import reusables

from pathlib import Path
import logging

from fastflix.plugins.common.helpers import generate_filters, Command
from fastflix.plugins.common.audio import build_audio

logger = logging.getLogger("fastflix")


class FlixError(Exception):
    pass


extension = "mkv"


@reusables.log_exception("fastflix", show_traceback=True)
def build(source, video_track, streams, start_time, format_info, duration, mode=7, qp=25, audio_tracks=(), **kwargs):
    file = Path(source)
    filters = generate_filters(**kwargs)

    fps_num, fps_denom = [
        int(x) for x in streams.video[0].get("avg_frame_rate", streams.video[0].r_frame_rate).split("/")
    ]
    bit_depth = 10 if streams.video[0].pix_fmt == "yuv420p10le" else 8
    crop = kwargs.get("crop")
    scale = kwargs.get("scale")

    if scale:
        width, height = (int(x) for x in scale.split(":"))
    else:
        height = int(streams.video[0].height)
        width = int(streams.video[0].width)
    assert height <= 2160
    assert width <= 4096

    if crop:
        crop_check = crop.split(":")
        try:
            assert crop_check[0] % 8 == 0
            assert crop_check[1] % 8 == 0
        except AssertionError:
            raise FlixError("CROP BAD: Video height and main_width must be divisible by 8")
    else:
        crop_height = height % 8
        crop_width = width % 8
        if crop_height or crop_width:
            raise FlixError("CROP BAD: Video height and main_width must be divisible by 8")

    intra_period = 1
    for i in range(1, 31):
        intra_period = (i * 8) - 1
        if (intra_period + 8) > (fps_num / fps_denom):
            break
    total_time = kwargs.get("duration", format_info.duration) - start_time
    frames = int(total_time * int(fps_num / fps_denom))
    command_1 = Command(
        (
            f'"{{ffmpeg}}" {f"-ss {start_time}" if start_time else ""} '
            f'-nostdin -i "{source}" -vframes {frames} -f rawvideo '
            f'{f"-vf {filters}" if filters else ""} '
            f'-pix_fmt {"yuv420p" if bit_depth == 8 else "yuv420p10le"} -an - | '
            f'"{{av1_aom}}" -intra-period {intra_period} -enc-mode {mode} -w {width} -h {height} '
            f"-bit-depth {bit_depth} -n {frames} -i stdin -q {qp} "
            f"-fps-num {fps_num} -fps-denom {fps_denom} -b <tempfile.1.ivf>"
        ),
        ["ffmpeg", "av1_aom"],
        False,
    )

    audio = build_audio(audio_tracks, audio_file_index=1)

    command_3 = Command(
        (
            f'"{{ffmpeg}}" -y '
            f'{f"-ss {start_time}" if start_time else ""} '
            f'{f"-t {duration - start_time}" if duration else ""} '
            f'-i "<tempfile.1.ivf>" -i "{file}" '
            f"-c copy -map 0:{video_track} "  # -af "aresample=async=1:min_hard_comp=0.100000:first_pts=0"
            f'{audio} "{{output}}"'
        ),
        ["ffmpeg", "output"],
        False,
    )

    return command_1, command_3



# -*- coding: utf-8 -*-
import reusables
import re

from fastflix.plugins.common.helpers import generate_filters, Command
from fastflix.plugins.common.audio import build_audio


def build(
    source,
    video_track,
    ffmpeg,
    bitrate=None,
    crf=None,
    start_time=0,
    duration=None,
    single_pass=False,
    quality="good",
    audio_tracks=(),
    speed=1,
    row_mt=0,
    force420=True,
    **kwargs,
):
    filters = generate_filters(**kwargs)
    audio = build_audio(audio_tracks)

    ending = "/dev/null"
    if reusables.win_based:
        ending = "NUL"

    beginning = (
        f'"{ffmpeg}" -y '
        f'-i "{source}" '
        f' {f"-ss {start_time}" if start_time else ""}  '
        f'{f"-t {duration}" if duration else ""} '
        f"-map 0:{video_track} "
        f"-c:v:0 libvpx-vp9 "
        f'{f"-vf {filters}" if filters else ""} '
        f'{"-pix_fmt yuv420p" if force420 else ""} '
        f'{"-row-mt 1" if row_mt else ""} '
        "-map_metadata -1 "
    )

    if not single_pass:
        beginning += '-passlogfile "<tempfile.1.log>" '

    beginning = re.sub("[ ]+", " ", beginning)

    if bitrate:
        command_1 = f"{beginning} -b:v {bitrate} -quality good -pass 1 -an -f webm {ending}"
        command_2 = f'{beginning} -b:v {bitrate}  -quality {quality} -speed {speed} -pass 2 {audio} "{{output}}"'

    elif crf:
        command_1 = f"{beginning} -b:v 0 -crf {crf} -quality good -pass 1 -an -f webm {ending}"
        command_2 = (
            f"{beginning} -b:v 0 -crf {crf} -quality {quality} -speed {speed} "
            f'{"-pass 2" if not single_pass else ""} {audio} "{{output}}"'
        )

    else:
        return []

    if crf and single_pass:
        return [Command(command_2, ["ffmpeg", "output"], False, name="Single pass CRF", exe="ffmpeg")]
    pass_type = "bitrate" if bitrate else "CRF"
    return [
        Command(command_1, ["ffmpeg", "output"], False, name=f"First pass {pass_type}", exe="ffmpeg"),
        Command(command_2, ["ffmpeg", "output"], False, name=f"Second pass {pass_type} ", exe="ffmpeg"),
    ]



# -*- coding: utf-8 -*-
import os
import sys
import importlib.machinery

try:
    # PyInstaller creates a temp folder and stores path in _MEIPASS
    # noinspection PyUnresolvedReferences
    base_path = sys._MEIPASS
    pyinstaller = True
except AttributeError:
    base_path = os.path.abspath(".")
    pyinstaller = False


from qtpy import QtWidgets, QtCore, QtGui

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)

main_width = 800


def message(msg):
    sm = QtWidgets.QMessageBox()
    sm.setText(msg)
    sm.setStandardButtons(QtWidgets.QMessageBox.Ok)
    sm.exec_()


def error_message(msg, details=None, traceback=False):
    em = QtWidgets.QMessageBox()
    em.setText(msg)
    if details:
        em.setDetailedText(details)
    elif traceback:
        import traceback

        em.setDetailedText(traceback.format_exc())
    em.setStandardButtons(QtWidgets.QMessageBox.Close)
    em.exec_()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import reusables

from pathlib import Path
import logging
import os

from fastflix.plugins.common.helpers import generate_filters, Loop, Command
from fastflix.plugins.common.audio import build_audio

logger = logging.getLogger("fastflix")


class FlixError(Exception):
    pass


extension = "mkv"


@reusables.log_exception("fastflix", show_traceback=True)
def build(
    source,
    video_track,
    stream_track,
    ffmpeg,
    av1,
    streams,
    start_time,
    duration,
    speed=7,
    segment_size=60,
    qp=25,
    pix_fmt="yuv420p10le",
    bitrate=None,
    audio_tracks=(),
    **kwargs,
):
    file = Path(source)

    filters = generate_filters(**kwargs)
    disable_hdr = kwargs.get("disable_hdr", False)

    fps_num, fps_denom = [
        int(x)
        for x in streams.video[stream_track].get("avg_frame_rate", streams.video[stream_track].r_frame_rate).split("/")
    ]
    bit_depth = 10 if pix_fmt == "yuv420p10le" and not disable_hdr else 8
    crop = kwargs.get("crop")
    scale = kwargs.get("scale")

    if scale:
        width, height = (int(x) for x in scale.split(":"))
    else:
        height = int(streams.video[stream_track].height)
        width = int(streams.video[stream_track].width)
        if crop:
            crop_check = crop.split(":")
            try:
                assert crop_check[0] % 8 == 0
                assert crop_check[1] % 8 == 0
            except AssertionError:
                raise FlixError("CROP BAD: Video height and main_width must be divisible by 8")
        else:
            crop_height = height % 8
            crop_width = width % 8
            if crop_height or crop_width:
                raise FlixError("CROP BAD: Video height and main_width must be divisible by 8")

    assert height <= 2160
    assert width <= 4096

    for track in audio_tracks:
        track.outdex -= 1

    audio = build_audio(audio_tracks, audio_file_index=0)
    audio_file = "<tempfile.7.mkv>"

    command_1 = Command(
        (
            f'"{ffmpeg}" -y '
            f'{f"-ss {start_time}" if start_time else ""} '
            f'{f"-t {duration - start_time}" if duration else ""} '
            f'-i "{source}" '
            f'{"-map_metadata -1" if kwargs.get("start_time") else ""} '
            f"-map 0:{video_track} -c copy -sc_threshold 0 "
            f"-reset_timestamps 1 -f segment -segment_time {segment_size} -an -sn -dn "
            f'"<tempdir.1>{os.sep}%04d{file.suffix}"'
        ),
        ["ffmpeg"],
        False,
        name="Segment movie into smaller files",
        exe="ffmpeg",
    )

    def func(tempfiles, tempdirs):
        return sorted(
            [(int(x.stem), x, Path(tempdirs["3"], f"{x.stem}.yuv")) for x in Path(tempdirs["1"]).iterdir()],
            key=lambda x: x[0],
        )

    loop_command_1 = (
        f'"{ffmpeg}" -y -i "<loop.1>" '
        f'-pix_fmt {"yuv420p10le" if bit_depth == 10 and not disable_hdr else "yuv420p"}'
        f' {f"-vf {filters}" if filters else ""} "<loop.2>"'
    )

    intra_period = 1
    for i in range(1, 31):
        intra_period = (i * 8) - 1
        if (intra_period + 8) > (fps_num / fps_denom):
            break
    logger.debug(f"setting intra-period to {intra_period} based of fps {float(fps_num / fps_denom):.2f}")

    quality = f"-rc 1 -tbr {bitrate}" if bitrate else f"--rc 0 -q {qp}"

    loop_command_2 = (
        f'"{av1}" --keyint {intra_period} --preset {speed} --input-depth {bit_depth} '
        # f'{"-hdr 1" if not disable_hdr and bit_depth == 10 else ""}'
        f" --fps-num {fps_num} --fps-denom {fps_denom} -w {width} -h {height} "
        f'{quality} -i "<loop.2>" -b "<tempdir.2>{os.sep}<loop.0>.ivf"'
    )

    loop_command_3 = f"echo file '<tempdir.2>{os.sep}<loop.0>.ivf' >> \"<tempfile.5.log>\""

    loop_command_4 = 'del /f "<loop.2>"'

    main_loop = Loop(
        name="Convert segmented input files into AV1 video files",
        condition=func,
        dirs=["2", "3"],
        files=[],
        commands=[
            Command(loop_command_1, ["ffmpeg"], False, exe="ffmpeg", name="Convert segment to raw YUV"),
            Command(loop_command_2, ["av1"], False, name="Convert YUV into AV1 binary IVF files"),
            Command(loop_command_3, [], False, name="Add new IVF file to ffmpeg concat list"),
            Command(loop_command_4, [], False, name="Remove large YUV temp file"),
        ],
    )

    if not audio_tracks:
        command_2 = Command(
            f'"{ffmpeg}" -y -safe 0 -f concat -i "<tempfile.5.log>" -reset_timestamps 1 -c copy "{{output}}"',
            ["ffmpeg"],
            False,
            exe="ffmpeg",
            name="Wrap IVF binary output into MKV container",
        )
        return command_1, main_loop, command_2

    no_audio_file = "<tempfile.6.mkv>"
    command_2 = Command(
        f'"{ffmpeg}" -y -safe 0 -f concat -i "<tempfile.5.log>" -reset_timestamps 1 -c copy "{no_audio_file}"',
        ["ffmpeg"],
        False,
        exe="ffmpeg",
        name="Add all the IVF files into a single MKV video",
    )

    #

    command_audio = Command(
        (
            f'"{ffmpeg}" -y '
            f'{f"-ss {start_time}" if start_time else ""} '
            f'{f"-t {duration - start_time}" if duration else ""} '
            f'-i "{source}" '
            f'{audio} "{audio_file}"'
        ),
        ["ffmpeg"],
        False,
        exe="ffmpeg",
        name="Split audio at proper time offsets into new file",
    )

    command_3 = Command(
        (
            f'"{ffmpeg}" -y '
            f'-i "{no_audio_file}" -i "{audio_file}" '
            f'-map_metadata -1 {"-shortest -reset_timestamps 1" if start_time or duration else ""} '
            f"-c copy -map 0:v -map 1:a "
            # -af "aresample=async=1:min_hard_comp=0.100000:first_pts=0"
            f'"{{output}}"'
        ),
        ["ffmpeg", "output"],
        False,
        exe="ffmpeg",
        name="Combine audio and video files into MKV container",
    )

    return command_1, main_loop, command_2, command_audio, command_3



# -*- coding: utf-8 -*-
from subprocess import run, PIPE, Popen, STDOUT
import logging
import os
from pathlib import Path

from box import Box, BoxError, BoxList

__all__ = ["FlixError", "ff_version", "Flix", "svt_av1_version"]

here = os.path.abspath(os.path.dirname(__file__))

logger = logging.getLogger("fastflix")


class FlixError(Exception):
    """This fastflix won't fly"""


def ff_version(ff, throw=True):
    res = Flix.execute(f'"{ff}" -version')
    if res.returncode != 0:
        if throw:
            raise FlixError(f'"{ff}" file not found')
        else:
            return False
    return res.stdout.decode("utf-8").split(" ", 4)[2]


def svt_av1_version(sv1_av1):
    """
    SVT AV1 Currently does not have a version associated with it's build so can only check for existence

    :param sv1_av1: Path to executable
    :return: True if SVT-AV1 else False
    """
    res = Flix.execute(f'"{sv1_av1}" -help')
    if res.returncode != 0:
        logger.warning(f"{sv1_av1} errored while checking version")
        return False
    try:
        lines = res.stdout.decode("utf-8").split("\n")
    except (ValueError, AttributeError):
        logger.warning(f"{sv1_av1} errored while decoding output")
        return False
    if len(lines) < 2:
        logger.warning(f"{sv1_av1} did not have enough output to check, must not be right")
        return False
    if lines[1].startswith("SVT-AV1"):
        return True
    logger.warning(f"{sv1_av1} not a SVT-AV1 file")
    return False


def guess_bit_depth(pix_fmt, color_primaries):
    eight = (
        "bgr0",
        "bgra",
        "gbrp",
        "gray",
        "monob",
        "monow",
        "nv12",
        "nv12m",
        "nv16",
        "nv20le",
        "nv21",
        "pal8",
        "rgb24",
        "rgb48le",
        "rgba",
        "rgba64le",
        "ya8",
        "yuv410p",
        "yuv411p",
        "yuv420p",
        "yuv422p",
        "yuv440p",
        "yuv444p",
        "yuva420p",
        "yuva422p",
        "yuva444p",
        "yuvj420p",
        "yuvj422p",
        "yuvj444p",
    )

    ten = ("yuv420p10le", "yuv422p10le", "yuv444p10le", "gbrp10le", "gray10le")

    twelve = ("yuv420p12le", "yuv422p12le", "yuv444p12le", "gbrp12le", "gray12le")

    if pix_fmt in eight:
        return 8
    if pix_fmt in ten:
        return 10
    if pix_fmt in twelve:
        return 12

    if color_primaries == "bt2020":
        return 10
    else:
        return 8


class Flix:
    def __init__(self, ffmpeg="ffmpeg", ffprobe="ffprobe", svt_av1="SvtAv1EncApp"):
        self.ffmpeg = ffmpeg
        self.ffprobe = ffprobe
        self.av1 = svt_av1
        self.config, self.filters = self.ffmpeg_configuration()

    def probe(self, file):
        command = f'"{self.ffprobe}" -v quiet -print_format json -show_format -show_streams "{file}"'
        logger.debug(f"running probe command: {command}")
        result = self.execute(command)
        try:
            return Box.from_json(result.stdout.decode("utf-8"))
        except BoxError:
            logger.error(f"Could not decode output: {result.stderr}")
            raise FlixError(result.stderr)

    def ffmpeg_configuration(self):
        output = self.execute(f'"{self.ffmpeg}" -version').stdout.decode("utf-8")
        config = []
        line_denote = "configuration: "
        for line in output.split("\n"):
            if line.startswith(line_denote):
                config = [x[9:].strip() for x in line[len(line_denote) :].split(" ") if x.startswith("--enable")]

        filter_output = self.execute(f'"{self.ffmpeg}" -hide_banner -filters').stdout.decode("utf-8")

        filters = []
        for i, line in enumerate(filter_output.split("\n")):
            if i < 8 or not line.strip():
                continue
            filters.append(line.strip().split(" ")[1])

        return config, filters

    def parse(self, file):
        data = self.probe(file)
        if "streams" not in data:
            raise FlixError("Not a video file")
        streams = Box({"video": [], "audio": [], "subtitle": [], "attachment": [], "data": []})

        for track in data.streams:
            if track.codec_type in streams:
                streams[track.codec_type].append(track)
            else:
                logger.error(f"Unknown codec: {track.codec_type}")

        for stream in streams.video:
            if "bits_per_raw_sample" in stream:
                stream.bit_depth = int(stream.bits_per_raw_sample)
            else:
                stream.bit_depth = guess_bit_depth(stream.pix_fmt, stream.get("color_primaries"))

        return streams, data.format

    @staticmethod
    def generate_filters(
        disable_hdr=False, scale_width=None, scale_height=None, crop=None, scale=None, scale_filter="lanczos"
    ):
        filter_list = []
        if crop:
            filter_list.append(f"crop={crop}")
        if scale:
            filter_list.append(f"scale={scale}:flags={scale_filter}")
        elif scale_width:
            filter_list.append(f"scale={scale_width}:-1:flags={scale_filter}")
        elif scale_height:
            filter_list.append(f"scale=-1:{scale_height}:flags={scale_filter}")

        if disable_hdr:
            filter_list.append(
                "zscale=t=linear:npl=100,format=gbrpf32le,zscale=p=bt709,tonemap=tonemap=hable:desat=0,"
                "zscale=t=bt709:m=bt709:r=tv,format=yuv420p"
            )

        return ",".join(filter_list)

    def generate_pallet_command(self, source, output, filters, video_track, start_time=0, duration=None):
        start = ""
        if duration:
            start = f"-ss {start_time} -t {duration}"

        return (
            f'"{self.ffmpeg}" {start} -i "{source}" -map 0:{video_track} '
            f'-vf "{f"{filters}," if filters else ""}palettegen" -y "{output}"'
        )

    def generate_gif_command(
        self,
        source,
        output,
        pallet_file,
        video_track,
        additional_tracks=(),
        start_time=0,
        duration=None,
        filters=None,
        fps=15,
        dither="sierra2_4a",
    ):
        start = ""
        if duration:
            start = f"-ss {start_time} -t {duration}"

        maps = ""
        for track in additional_tracks:
            maps += f" -map 0:{track} "

        gif_filters = f"fps={fps:.2f}"
        if filters:
            gif_filters += f",{filters}"

        return (
            f'"{self.ffmpeg}" {start} -i "{source}" -i "{pallet_file}" -map 0:{video_track} '
            f'-lavfi "{gif_filters} [x]; [x][1:v] paletteuse=dither={dither}" -y "{output}" '
        )

    def generate_x265_command(
        self,
        source,
        output,
        video_track,
        audio_track=None,
        additional_tracks=(),
        start_time=0,
        duration=None,
        crf=20,
        preset="medium",
        disable_hdr=False,
        scale_width=None,
        scale_height=None,
        keep_subtitles=False,
        crop=None,
        scale=None,
    ):
        start = ""
        if duration:
            start = f"-ss {start_time} -t {duration}"

        maps = ""
        for track in additional_tracks:
            maps += f" -map 0:{track} "

        filter_list = []

        if disable_hdr:
            filter_list.append(
                "zscale=t=linear:npl=100,format=gbrpf32le,zscale=p=bt709,tonemap=tonemap=hable:desat=0,"
                "zscale=t=bt709:m=bt709:r=tv,format=yuv420p"
            )

        if scale:
            filter_list.append(f"scale={scale}")
        elif scale_width:
            filter_list.append(f"scale={scale_width}:-1")
        elif scale_height:
            filter_list.append(f"scale=-1:{scale_height}")

        if crop:
            filter_list.append(f"crop={crop}")

        filters = ",".join(filter_list)

        return (
            f'"{self.ffmpeg}" -loglevel error {start} -i "{source}" '
            f"-c:v libx265 -preset {preset} -x265-params log-level=error:crf={crf} -pix_fmt yuv420p "
            f'{"-map_metadata -1 -write_tmcd 0 -shortest" if start else ""} {f"-vf {filters}" if filters else ""} '
            f'-map 0:{video_track} {"-an" if audio_track is None else f"-map 0:{audio_track}"} {maps} '
            f'{"-map 0:s" if keep_subtitles else "-sn"} '
            # -filter_complex "[0:v:0][0:3]overlay"
            f' -y "{output}"'
        )

    def generate_av1_command(
        self,
        source,
        output,
        video_track,
        audio_track=None,
        additional_tracks=(),
        start_time=0,
        duration=None,
        crf=20,
        disable_hdr=False,
        scale_width=None,
        scale_height=None,
        keep_subtitles=False,
        crop=None,
    ):
        start = ""
        if start_time:
            start += "-ss {start_time}"
        if duration:
            start += f" -t {duration} -write_tmcd 0"

        maps = ""
        for track in additional_tracks:
            maps += f" -map 0:{track} "

        filter_list = []

        if disable_hdr:
            filter_list.append(
                "zscale=t=linear:npl=100,format=gbrpf32le,zscale=p=bt709,tonemap=tonemap=hable:desat=0,"
                "zscale=t=bt709:m=bt709:r=tv,format=yuv420p"
            )

        if scale_width:
            filter_list.append(f"scale={scale_width}:-1")
        elif scale_height:
            filter_list.append(f"scale=-1:{scale_height}")

        if crop:
            filter_list.append(f"crop={crop}")

        filters = ",".join(filter_list)

        return (
            f'"{self.ffmpeg}" -loglevel error -i "{source}" {start} '
            f"-c:v libaom-av1 -b:v 0 -strict experimental -crf {crf} -pix_fmt yuv420p "
            f'{"-map_metadata -1" if start else ""} {f"-vf {filters}" if filters else ""} '
            f'-map 0:{video_track} {"-an" if audio_track is None else f"-map 0:{audio_track}"} {maps} '
            f'{"-map 0:s" if keep_subtitles else "-sn"} '
            f' -y "{output}"'
        )

    def generate_thumbnail_command(self, source, output, video_track, start_time=0, filters=None):
        start = ""
        if start_time:
            start = f"-ss {start_time}"
        return (
            f'"{self.ffmpeg}" {start} -loglevel error -i "{source}" '
            f' -vf {filters + "," if filters else ""}scale="min(320\\,iw):-1" '
            f"-map 0:{video_track} -an -y -map_metadata -1 "
            f'-vframes 1 "{output}"'
        )

    @staticmethod
    def execute(command):
        logger.debug(f"running command: {command}")
        return run(command, stdout=PIPE, stderr=PIPE, stdin=PIPE, shell=True)

    def video_split_command(self, source, video_track=0, start_time=0, duration=None, build_dir=".", segment_size=60):
        start = ""
        if start_time:
            start += f"-ss {start_time}"
        if duration:
            start += f" -t {duration}"
        src = Path(source)
        out = Path(build_dir, f"%04d{src.suffix}")

        return (
            f'"{self.ffmpeg}" -loglevel error {start} -i "{source}" '
            f'{"-map_metadata -1" if start else ""} -map 0:{video_track} -c copy -sc_threshold 0 '
            f'-reset_timestamps 1 -f segment -segment_time {segment_size} -an -sn -dn "{out}"'
        )

    def yuv_command(self, source, output, bit_depth=8, crop=None, scale=None):
        assert str(output).endswith(("yuv", "y4m"))

        filter_list = []
        if crop:
            filter_list.append(f"crop={crop}")
        if scale:
            filter_list.append(f"scale={scale}")

        filters = ",".join(filter_list) if filter_list else ""

        return (
            f'"{self.ffmpeg}" -loglevel error -i "{source}" -c:v rawvideo '
            f'-pix_fmt {"yuv420p10le" if bit_depth == 10 else "yuv420p"}'
            f' {f"-vf {filters}" if filters else ""} "{output}"'
        )

    def svt_av1_command(self, source, output, height, width, fps_num, fps_denom, crf=30, mode=3, bit_depth=8):
        if not self.av1:
            raise FlixError("AV1 not defined")
        intra_period = 1
        for i in range(1, 31):
            intra_period = (i * 8) - 1
            if (intra_period + 8) > (fps_num / fps_denom):
                break
        logger.debug(f"setting intra-period to {intra_period} based of fps {float(fps_num / fps_denom):.2f}")
        return (
            f'"{self.av1}" -intra-period {intra_period} -enc-mode {mode} -bit-depth {bit_depth} '
            f"-w {width} -h {height} -fps-num {fps_num} -fps-denom {fps_denom} "
            f'-q {crf} -i "{source}" -b "{output}"'
        )

    def combine_command(self, videos, output, build_dir="."):
        import uuid

        file_list = os.path.abspath(os.path.join(build_dir, uuid.uuid4().hex))
        with open(file_list, "w") as f:
            f.write("\n".join(["file {}".format(str(video).replace("\\", "\\\\")) for video in videos]))
        return f'"{self.ffmpeg}" -safe 0 -f concat -i "{file_list}" -reset_timestamps 1 -c copy "{output}"'

    def extract_audio_command(
        self, video, start_time, duration, output, audio_track=0, audio_format="adts", convert=False
    ):
        start = ""
        if start_time:
            start += f"-ss {start_time}"
        if duration:
            start += f" -t {duration}"

        options = f"-acodec copy -f {audio_format}" if not convert else f"-acodec libvorbis"

        return f'"{self.ffmpeg}" {start} -i "{video}" ' f'-vn {options} -map 0:{audio_track} "{output}"'

    def add_audio_command(self, video_source, audio_source, output, video_track=0, audio_track=0):
        # -shortest ?
        # https://videoblerg.wordpress.com/2017/11/10/ffmpeg-and-how-to-use-it-wrong/
        return (
            f'"{self.ffmpeg}" -i "{video_source}" -i "{audio_source}" '
            f'-c copy -map 0:{video_track} -af "aresample=async=1:min_hard_comp=0.100000:first_pts=0" '
            f'-map 1:{audio_track} "{output}"'
        )

    def get_audio_encoders(self):
        cmd = Popen(
            f'"{self.ffmpeg}" -hide_banner -encoders', shell=True, stderr=STDOUT, stdout=PIPE, universal_newlines=True
        )
        encoders = []
        start_line = " ------"
        started = False
        for line in cmd.stdout:
            if started:
                if line.strip().startswith("A"):
                    encoders.append(line.strip().split(" ")[1])
            elif line.startswith(start_line):
                started = True
        return encoders

    def parse_hdr_details(self, video_source, video_track=0):
        command = (
            f'"{self.ffprobe}" -select_streams v:{video_track} -print_format json -show_frames '
            '-read_intervals "%+#1" '
            '-show_entries "frame=color_space,color_primaries,color_transfer,side_data_list,pix_fmt" '
            f'-i "{video_source}"'
        )

        result = run(command, shell=True, stdout=PIPE, stderr=PIPE)
        try:
            data = Box.from_json(result.stdout.decode("utf-8"), default_box=True, default_box_attr=None)
        except BoxError:
            # Could not parse details
            logger.error(
                "COULD NOT PARSE FFPROBE HDR METADATA, PLEASE OPEN ISSUE WITH THESE DETAILS:"
                f"\nSTDOUT: {result.stdout.decode('utf-8')}\nSTDERR: {result.stderr.decode('utf-8')}"
            )
            return
        if "frames" not in data or not len(data.frames):
            return
        data = data.frames[0]
        if not data.get("side_data_list"):
            return

        master_display = None
        cll = None

        def s(a, v):
            return int(a.get(v, "0").split("/")[0])

        for item in data["side_data_list"]:
            if item.side_data_type == "Mastering display metadata":
                master_display = Box(
                    red=f"({s(item, 'red_x')},{s(item, 'red_y')})",
                    green=f"({s(item, 'green_x')},{s(item, 'green_y')})",
                    blue=f"({s(item, 'blue_x')},{s(item, 'blue_y')})",
                    white=f"({s(item, 'white_point_x')},{s(item, 'white_point_y')})",
                    luminance=f"({s(item, 'max_luminance')},{s(item, 'min_luminance')})",
                )
            if item.side_data_type == "Content light level metadata":
                cll = f"{item.max_content},{item.max_average}"

        return Box(
            pix_fmt=data.pix_fmt,
            color_space=data.color_space,
            color_primaries=data.color_primaries,
            color_transfer=data.color_transfer,
            master_display=master_display,
            cll=cll,
        )



# -*- coding: utf-8 -*-
import logging

from box import Box

from qtpy import QtWidgets, QtCore, QtGui

logger = logging.getLogger("fastflix")

recommended_bitrates = [
    "100k   (320x240p @ 24,25,30)",
    "200k   (640x360p @ 24,25,30)",
    "400k   (640x480p @ 24,25,30)",
    "800k  (1280x720p @ 24,25,30)",
    "1200k (1280x720p @ 50,60)",
    "1200k (1920x1080p @ 24,25,30)",
    "2000k (1920x1080p @ 50,60)",
    "4000k (2560x1440p @ 24,25,30)",
    "6000k (2560x1440p @ 50,60)",
    "9000k (3840x2160p @ 24,25,30)",
    "13000k (3840x2160p @ 50,60)",
    "Custom",
]

recommended_crfs = ["34", "32", "30", "28", "26", "24", "22", "20", "Custom"]


class AV1(QtWidgets.QWidget):
    def __init__(self, parent, main):
        super(AV1, self).__init__(parent)
        self.main = main

        grid = QtWidgets.QGridLayout()

        # grid.addWidget(QtWidgets.QLabel("FFMPEG libaom-av1_aom"), 0, 0)

        self.widgets = Box(fps=None, remove_hdr=None, mode=None)

        self.mode = "CRF"

        grid.addLayout(self.init_remove_hdr(), 0, 0, 1, 2)
        grid.addLayout(self.init_cpu_used(), 1, 0, 1, 2)
        grid.addLayout(self.init_row_mt(), 2, 0, 1, 2)
        grid.addLayout(self.init_tile_columns(), 3, 0, 1, 2)
        grid.addLayout(self.init_tile_rows(), 4, 0, 1, 2)
        grid.addLayout(self.init_modes(), 0, 2, 3, 3)

        grid.addWidget(QtWidgets.QWidget(), 5, 0)
        grid.setRowStretch(5, 1)
        guide_label = QtWidgets.QLabel(
            f"<a href='https://trac.ffmpeg.org/wiki/Encode/AV1'>FFMPEG AV1 Encoding Guide</a>"
        )
        guide_label.setAlignment(QtCore.Qt.AlignBottom)
        guide_label.setOpenExternalLinks(True)
        grid.addWidget(guide_label, 9, 0, -1, 1)

        self.setLayout(grid)
        self.hide()

    def init_cpu_used(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("CPU Used")
        label.setToolTip("Quality/Speed ratio modifier (defaults to 1)")
        layout.addWidget(label)
        self.widgets.cpu_used = QtWidgets.QComboBox()
        self.widgets.cpu_used.addItems([str(x) for x in range(0, 9)])
        self.widgets.cpu_used.setCurrentIndex(1)
        self.widgets.cpu_used.currentIndexChanged.connect(lambda: self.main.build_commands())
        layout.addWidget(self.widgets.cpu_used)
        return layout

    def init_row_mt(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Row Multi-Threading")
        label.setToolTip("Enable row based multi-threading")
        layout.addWidget(label)
        self.widgets.row_mt = QtWidgets.QComboBox()
        self.widgets.row_mt.addItems(["default", "enabled", "disabled"])
        self.widgets.row_mt.setCurrentIndex(0)
        self.widgets.row_mt.currentIndexChanged.connect(lambda: self.main.build_commands())
        layout.addWidget(self.widgets.row_mt)
        return layout

    def init_tile_columns(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Tile Columns")
        label.setToolTip("Log2 of number of tile columns to use")
        layout.addWidget(label)
        self.widgets.tile_columns = QtWidgets.QComboBox()
        self.widgets.tile_columns.addItems([str(x) for x in range(-1, 7)])
        self.widgets.tile_columns.setCurrentIndex(0)
        self.widgets.tile_columns.currentIndexChanged.connect(lambda: self.main.build_commands())
        layout.addWidget(self.widgets.tile_columns)
        return layout

    def init_tile_rows(self):
        layout = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel("Tile Rows")
        label.setToolTip("Log2 of number of tile rows to use")
        layout.addWidget(label)
        self.widgets.tile_rows = QtWidgets.QComboBox()
        self.widgets.tile_rows.addItems([str(x) for x in range(-1, 7)])
        self.widgets.tile_rows.setCurrentIndex(0)
        self.widgets.tile_rows.currentIndexChanged.connect(lambda: self.main.build_commands())
        layout.addWidget(self.widgets.tile_rows)
        return layout

    def init_remove_hdr(self):
        layout = QtWidgets.QHBoxLayout()
        self.remove_hdr_label = QtWidgets.QLabel("Remove HDR")
        self.remove_hdr_label.setToolTip(
            "Convert BT2020 colorspace into bt709\n " "WARNING: This will take much longer and result in a larger file"
        )
        layout.addWidget(self.remove_hdr_label)
        self.widgets.remove_hdr = QtWidgets.QComboBox()
        self.widgets.remove_hdr.addItems(["No", "Yes"])
        self.widgets.remove_hdr.setCurrentIndex(0)
        self.widgets.remove_hdr.setDisabled(True)
        self.widgets.remove_hdr.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.remove_hdr)
        return layout

    def init_modes(self):
        layout = QtWidgets.QGridLayout()
        crf_group_box = QtWidgets.QGroupBox()
        crf_group_box.setFixedHeight(40)
        crf_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        crf_box_layout = QtWidgets.QHBoxLayout()
        bitrate_group_box = QtWidgets.QGroupBox()
        bitrate_group_box.setFixedHeight(40)
        bitrate_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        bitrate_box_layout = QtWidgets.QHBoxLayout()
        # rotation_dir = Path(base_path, 'data', 'rotations')
        # group_box.setStyleSheet("QGroupBox{padding-top:15px; margin-top:-15px; padding-bottom:-5px}")
        self.widgets.mode = QtWidgets.QButtonGroup()
        self.widgets.mode.buttonClicked.connect(self.set_mode)

        bitrate_radio = QtWidgets.QRadioButton("Bitrate")
        self.widgets.mode.addButton(bitrate_radio)
        self.widgets.bitrate = QtWidgets.QComboBox()
        self.widgets.bitrate.addItems(recommended_bitrates)
        self.widgets.bitrate.setCurrentIndex(6)
        self.widgets.bitrate.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_bitrate = QtWidgets.QLineEdit("3000")
        self.widgets.custom_bitrate.setFixedWidth(100)
        self.widgets.custom_bitrate.setDisabled(True)
        self.widgets.custom_bitrate.textChanged.connect(lambda: self.main.build_commands())
        bitrate_box_layout.addWidget(bitrate_radio)
        bitrate_box_layout.addWidget(self.widgets.bitrate)
        bitrate_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        bitrate_box_layout.addWidget(self.widgets.custom_bitrate)

        crf_radio = QtWidgets.QRadioButton("CRF")
        crf_radio.setChecked(True)
        self.widgets.mode.addButton(crf_radio)

        self.widgets.crf = QtWidgets.QComboBox()
        self.widgets.crf.addItems(recommended_crfs)
        self.widgets.crf.setCurrentIndex(2)
        self.widgets.crf.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_crf = QtWidgets.QLineEdit("30")
        self.widgets.custom_crf.setFixedWidth(100)
        self.widgets.custom_crf.setDisabled(True)
        self.widgets.custom_crf.textChanged.connect(lambda: self.main.build_commands())
        crf_box_layout.addWidget(crf_radio)
        crf_box_layout.addWidget(self.widgets.crf)
        crf_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        crf_box_layout.addWidget(self.widgets.custom_crf)

        bitrate_group_box.setLayout(bitrate_box_layout)
        crf_group_box.setLayout(crf_box_layout)

        layout.addWidget(crf_group_box, 0, 0)
        layout.addWidget(bitrate_group_box, 1, 0)
        return layout

    def mode_update(self):
        self.widgets.custom_crf.setDisabled(self.widgets.crf.currentText() != "Custom")
        self.widgets.custom_bitrate.setDisabled(self.widgets.bitrate.currentText() != "Custom")
        self.main.build_commands()

    def get_settings(self):
        conversions = {"default": None, "enabled": 1, "disabled": 0}
        settings = Box(
            disable_hdr=bool(self.widgets.remove_hdr.currentIndex()),
            cpu_used=self.widgets.cpu_used.currentText(),
            row_mt=conversions[self.widgets.row_mt.currentText()],
            tile_rows=self.widgets.tile_rows.currentText(),
            tile_columns=self.widgets.tile_columns.currentText(),
        )

        if self.mode == "CRF":
            settings.crf = int(self.widgets.crf.currentText().split(" ", 1)[0])
        else:
            settings.bitrate = self.widgets.bitrate.currentText().split(" ", 1)[0]
        return settings

    def new_source(self):
        if not self.main.streams:
            return
        if "zcale" not in self.main.flix.filters:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#777}")
            self.remove_hdr_label.setToolTip("cannot remove HDR, zcale filter not in current version of FFmpeg")
            logger.warning("zcale filter not detected in current version of FFmpeg, cannot remove HDR")
        elif self.main.streams["video"][self.main.video_track].get("color_space", "").startswith("bt2020"):
            self.widgets.remove_hdr.setDisabled(False)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")
        else:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")

    def set_mode(self, x):
        self.mode = x.text()
        self.main.build_commands()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Chris Griffith"

name = "VP9"
requires = "libvpx"

video_extension = "webm"
video_dimension_divisor = 1

enable_subtitles = True
enable_audio = True
enable_attachments = True

audio_formats = ["libopus", "libvorbis"]

from fastflix.plugins.vp9.command_builder import build
from fastflix.plugins.vp9.settings_panel import VP9 as settings_panel



# -*- coding: utf-8 -*-
import reusables
import re

from fastflix.plugins.common import helpers
from fastflix.plugins.common.audio import build_audio


def build(
    source,
    video_track,
    ffmpeg,
    bitrate=None,
    crf=None,
    start_time=0,
    duration=None,
    audio_tracks=(),
    row_mt=None,
    cpu_used="1",
    tile_columns="-1",
    tile_rows="-1",
    **kwargs,
):
    filters = helpers.generate_filters(**kwargs)
    audio = build_audio(audio_tracks)

    ending = "/dev/null"
    if reusables.win_based:
        ending = "NUL"

    beginning = (
        f'"{ffmpeg}" -y '
        f' {f"-ss {start_time}" if start_time else ""}  '
        f'{f"-t {duration}" if duration else ""} '
        f'-i "{source}" '
        f"-map 0:{video_track} "
        f"-c:v:0 libaom-av1 -strict experimental "
        f'{f"-vf {filters}" if filters else ""} '
        f"-cpu-used {cpu_used} "
        f"-tile-rows {tile_rows} "
        f"-tile-columns {tile_columns} "
        "-map_metadata -1 "
    )

    if row_mt is not None:
        beginning += f"-row-mt {row_mt} "

    beginning = re.sub("[ ]+", " ", beginning)

    if bitrate:
        command_1 = f'{beginning} -passlogfile "<tempfile.1.log>" -b:v {bitrate} -pass 1 -an -f matroska {ending}'
        command_2 = f'{beginning} -passlogfile "<tempfile.1.log>" -b:v {bitrate} -pass 2 {audio} "{{output}}"'
        return [
            helpers.Command(command_1, ["ffmpeg", "output"], False, name="First Pass bitrate"),
            helpers.Command(command_2, ["ffmpeg", "output"], False, name="Second Pass bitrate"),
        ]
    elif crf:
        command_1 = f'{beginning} -b:v 0 -crf {crf} {audio} "{{output}}"'
        return [helpers.Command(command_1, ["ffmpeg", "output"], False, name="Single Pass CRF")]



# -*- coding: utf-8 -*-
import logging

from box import Box

from qtpy import QtWidgets, QtCore, QtGui

logger = logging.getLogger("fastflix")

recommended_bitrates = [
    "150000   (320x240p @ 24,25,30)",
    "276000   (640x360p @ 24,25,30)",
    "512000   (640x480p @ 24,25,30)",
    "1024000  (1280x720p @ 24,25,30)",
    "1800000 (1280x720p @ 50,60)",
    "1800000 (1920x1080p @ 24,25,30)",
    "3000000 (1920x1080p @ 50,60)",
    "6000000 (2560x1440p @ 24,25,30)",
    "9000000 (2560x1440p @ 50,60)",
    "12000000 (3840x2160p @ 24,25,30)",
    "18000000 (3840x2160p @ 50,60)",
    "Custom",
]

recommended_qp = ["24 - recommended", "30 - standard", '50 - "I\'m just testing to see if this works"', "Custom"]


class SVT_AV1(QtWidgets.QWidget):
    def __init__(self, parent, main):
        super(SVT_AV1, self).__init__(parent)
        self.main = main
        grid = QtWidgets.QGridLayout()

        self.widgets = Box(fps=None, remove_hdr=None, mode=None, segment_size=None)

        self.mode = "QP"

        grid.addLayout(self.init_remove_hdr(), 1, 0, 1, 2)
        grid.addLayout(self.init_speed(), 0, 0, 1, 2)

        grid.addLayout(self.init_modes(), 0, 2, 4, 4)
        grid.addLayout(self.init_segment_size(), 3, 0, 1, 2)

        grid.addWidget(QtWidgets.QWidget(), 5, 0)
        grid.setRowStretch(5, 1)
        guide_label = QtWidgets.QLabel(f"<a href='https://github.com/OpenVisualCloud/SVT-AV1'>SVT-AV1 Github</a>")
        guide_label.setAlignment(QtCore.Qt.AlignBottom)
        guide_label.setOpenExternalLinks(True)
        grid.addWidget(guide_label, 9, 0, -1, 1)
        self.setLayout(grid)
        self.hide()

    # def init_fps(self):
    #     layout = QtWidgets.QHBoxLayout()
    #     layout.addWidget(QtWidgets.QLabel("FPS"))
    #     self.widgets.fps = QtWidgets.QComboBox()
    #     self.widgets.fps.addItems([str(x) for x in range(1, 31)])
    #     self.widgets.fps.setCurrentIndex(14)
    #     self.widgets.fps.currentIndexChanged.connect(lambda: self.main.build_commands())
    #     layout.addWidget(self.widgets.fps)
    #     return layout

    def init_remove_hdr(self):
        layout = QtWidgets.QHBoxLayout()
        self.remove_hdr_label = QtWidgets.QLabel("Remove HDR")
        self.remove_hdr_label.setToolTip(
            "Convert BT2020 colorspace into bt709\n " "WARNING: This will take much longer and result in a larger file"
        )
        layout.addWidget(self.remove_hdr_label)
        self.widgets.remove_hdr = QtWidgets.QComboBox()
        self.widgets.remove_hdr.addItems(["No", "Yes"])
        self.widgets.remove_hdr.setCurrentIndex(0)
        self.widgets.remove_hdr.setDisabled(True)
        self.widgets.remove_hdr.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.remove_hdr)
        return layout

    def init_speed(self):
        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(QtWidgets.QLabel("Speed"))
        self.widgets.speed = QtWidgets.QComboBox()
        self.widgets.speed.addItems([str(x) for x in range(9)])
        self.widgets.speed.setCurrentIndex(7)
        self.widgets.speed.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.speed)
        return layout

    def init_segment_size(self):
        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(QtWidgets.QLabel("Segment Size (seconds)"))
        self.widgets.segment_size = QtWidgets.QComboBox()
        self.widgets.segment_size.addItems(["10", "30", "60", "90", "120", "240"])
        self.widgets.segment_size.setCurrentIndex(2)
        self.widgets.segment_size.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.segment_size)
        return layout

    def init_modes(self):
        layout = QtWidgets.QGridLayout()
        qp_group_box = QtWidgets.QGroupBox()
        qp_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        qp_box_layout = QtWidgets.QHBoxLayout()

        # rotation_dir = Path(base_path, 'data', 'rotations')
        # group_box.setStyleSheet("QGroupBox{padding-top:15px; margin-top:-15px; padding-bottom:-5px}")
        self.widgets.mode = QtWidgets.QButtonGroup()
        self.widgets.mode.buttonClicked.connect(self.set_mode)

        bitrate_group_box = QtWidgets.QGroupBox()
        bitrate_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        bitrate_box_layout = QtWidgets.QHBoxLayout()
        bitrate_radio = QtWidgets.QRadioButton("Bitrate")
        bitrate_radio.setFixedWidth(80)
        self.widgets.mode.addButton(bitrate_radio)
        self.widgets.bitrate = QtWidgets.QComboBox()
        self.widgets.bitrate.setFixedWidth(250)
        self.widgets.bitrate.addItems(recommended_bitrates)
        self.widgets.bitrate.setCurrentIndex(6)
        self.widgets.bitrate.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_bitrate = QtWidgets.QLineEdit("3000")
        self.widgets.custom_bitrate.setFixedWidth(100)
        self.widgets.custom_bitrate.setDisabled(True)
        self.widgets.custom_bitrate.textChanged.connect(lambda: self.main.build_commands())
        bitrate_box_layout.addWidget(bitrate_radio)
        bitrate_box_layout.addWidget(self.widgets.bitrate)
        bitrate_box_layout.addStretch()
        bitrate_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        bitrate_box_layout.addWidget(self.widgets.custom_bitrate)

        qp_radio = QtWidgets.QRadioButton("QP")
        qp_radio.setChecked(True)
        qp_radio.setFixedWidth(80)
        self.widgets.mode.addButton(qp_radio)

        self.widgets.qp = QtWidgets.QComboBox()
        self.widgets.qp.setFixedWidth(250)
        self.widgets.qp.addItems(recommended_qp)
        self.widgets.qp.setCurrentIndex(0)
        self.widgets.qp.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_qp = QtWidgets.QLineEdit("30")
        self.widgets.custom_qp.setFixedWidth(100)
        self.widgets.custom_qp.setDisabled(True)
        self.widgets.custom_qp.textChanged.connect(lambda: self.main.build_commands())
        qp_box_layout.addWidget(qp_radio)
        qp_box_layout.addWidget(self.widgets.qp)
        qp_box_layout.addStretch()
        qp_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        qp_box_layout.addWidget(self.widgets.custom_qp)

        bitrate_group_box.setLayout(bitrate_box_layout)
        qp_group_box.setLayout(qp_box_layout)

        bitrate_group_box.setLayout(bitrate_box_layout)
        qp_group_box.setLayout(qp_box_layout)

        layout.addWidget(qp_group_box, 0, 0)
        layout.addWidget(bitrate_group_box, 1, 0)
        return layout

    def mode_update(self):
        self.widgets.custom_qp.setDisabled(self.widgets.qp.currentText() != "Custom")
        self.widgets.custom_bitrate.setDisabled(self.widgets.bitrate.currentText() != "Custom")
        self.main.build_commands()

    def get_settings(self):
        settings = Box(
            disable_hdr=bool(self.widgets.remove_hdr.currentIndex()),
            speed=self.widgets.speed.currentText(),
            segment_size=int(self.widgets.segment_size.currentText()),
        )
        if self.mode == "QP":
            qp = self.widgets.qp.currentText()
            settings.qp = int(qp.split(" ", 1)[0]) if qp.lower() != "custom" else self.widgets.custom_qp.text()
        else:
            bitrate = self.widgets.bitrate.currentText()
            settings.bitrate = bitrate.split(" ", 1)[0] if bitrate.lower() != "custom" else self.widgets.bitrate.text()
        return settings

    def new_source(self):
        if not self.main.streams:
            return
        if "zcale" not in self.main.flix.filters:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#777}")
            self.remove_hdr_label.setToolTip("cannot remove HDR, zcale filter not in current version of FFmpeg")
            logger.warning("zcale filter not detected in current version of FFmpeg, cannot remove HDR")
        elif self.main.streams["video"][self.main.video_track].get("color_space", "").startswith("bt2020"):
            self.widgets.remove_hdr.setDisabled(False)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")
        else:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")

    def set_mode(self, x):
        self.mode = x.text()
        self.main.build_commands()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pathlib import Path
import pkg_resources

from qtpy import QtCore, QtWidgets, QtGui
from fastflix.widgets.main import Main
from fastflix.widgets.about import About
from fastflix.widgets.logs import Logs
from fastflix.widgets.settings import Settings


class Container(QtWidgets.QMainWindow):
    def __init__(self, data_path, work_path, config_file, **kwargs):
        super(Container, self).__init__()
        self.logs = Logs()
        self.about = None
        self.init_menu()
        self.config_file = config_file
        self.main = Main(self, data_path, work_path, **kwargs)
        self.setCentralWidget(self.main)
        self.setMinimumSize(1200, 600)
        my_data = str(Path(pkg_resources.resource_filename(__name__, f"../data/icon.ico")).resolve())
        icon = QtGui.QIcon(my_data)
        self.setWindowIcon(icon)
        self.setWindowIcon(icon)

    def init_menu(self):
        menubar = self.menuBar()

        file_menu = menubar.addMenu("&File")

        setting_action = QtWidgets.QAction(
            self.style().standardIcon(QtWidgets.QStyle.SP_FileDialogListView), "&Settings", self
        )
        setting_action.setShortcut("Ctrl+S")
        setting_action.triggered.connect(self.show_setting)

        exit_action = QtWidgets.QAction(
            self.style().standardIcon(QtWidgets.QStyle.SP_DialogCancelButton), "&Exit", self
        )
        exit_action.setShortcut(QtGui.QKeySequence("Ctrl+Q"))
        exit_action.setStatusTip("Exit application")
        exit_action.triggered.connect(self.close)

        file_menu.addAction(setting_action)
        file_menu.addSeparator()
        file_menu.addAction(exit_action)

        about_action = QtWidgets.QAction(
            self.style().standardIcon(QtWidgets.QStyle.SP_FileDialogInfoView), "&About", self
        )
        about_action.triggered.connect(self.show_about)

        log_action = QtWidgets.QAction(
            self.style().standardIcon(QtWidgets.QStyle.SP_FileDialogDetailedView), "View &Logs", self
        )
        log_action.triggered.connect(self.show_logs)

        report_action = QtWidgets.QAction("Report &Issue", self)
        report_action.triggered.connect(self.open_issues)

        help_menu = menubar.addMenu("&Help")
        help_menu.addAction(log_action)
        help_menu.addAction(report_action)
        help_menu.addAction(about_action)

    def show_about(self):
        self.about = About()
        self.about.show()

    def show_setting(self):
        self.setting = Settings(self.config_file, self.main)
        self.setting.show()

    def show_logs(self):
        self.logs.show()

    def open_issues(self):
        QtGui.QDesktopServices.openUrl(QtCore.QUrl("https://github.com/cdgriffith/FastFlix/issues"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import re
import tempfile
from pathlib import Path
from uuid import uuid4
from subprocess import Popen, PIPE, STDOUT
import os
import signal

import reusables

from qtpy import QtCore, QtWidgets, QtGui

logger = logging.getLogger("fastflix")

__all__ = ["CommandRunner"]

white_detect = re.compile(r"^\s+")


class CommandRunner(QtCore.QThread):
    def __init__(self, parent, command_list, work_dir):
        super().__init__(parent)
        self.tempdir = tempfile.TemporaryDirectory(prefix="temp_", dir=work_dir)
        self.app = parent
        self.command_list = command_list
        self.process = None
        self.killed = False
        self.re_tempfile = re.compile(r"<tempfile\.(\d+)\.(\w+)>")
        self.re_tempdir = re.compile(r"<tempdir\.(\d+)>")
        self.temp_files = {}
        self.temp_dirs = {}

    def replace_temps(self, command):
        file_numbers = set(self.re_tempfile.findall(command))
        for num, ext in file_numbers:
            if num not in self.temp_files:
                self.temp_files[num] = Path(self.tempdir.name, f"{uuid4().hex}.{ext}")
            command = command.replace(f"<tempfile.{num}.{ext}>", str(self.temp_files[num]))
        for num in set(self.re_tempdir.findall(command)):
            if num not in self.temp_dirs:
                self.temp_dirs[num] = Path(tempfile.mkdtemp(prefix=f"{num}_", dir=self.tempdir.name))
            command = command.replace(f"<tempdir.{num}>", str(self.temp_dirs[num]))
        return command

    def loop_creates(self, dirs, files):
        for num, ext in files:
            if num not in self.temp_files:
                self.temp_files[num] = Path(self.tempdir.name, f"{uuid4().hex}.{ext}")
        for num in dirs:
            if num not in self.temp_dirs:
                self.temp_dirs[num] = Path(tempfile.mkdtemp(prefix=f"{num}_", dir=self.tempdir.name))

    @staticmethod
    def replace_loop_args(command, index, items):
        command = command.replace("<loop.index>", str(index))
        for i in range(len(items)):
            command = command.replace(f"<loop.{i}>", str(items[i]))
        return command

    def run_command(self, command, command_type=None):
        command = self.replace_temps(command)
        logger.info(f"Running command: {command}")
        self.process = self.start_exec(command)
        if not command_type:
            line_wait = False
            line = ""
            while True:
                char = self.process.stdout.read(1)
                if char == "" and self.process.poll() is not None:
                    logger.info(line)
                    break
                if char != "":
                    if char in ("\r", "\n"):
                        logger.info(line)
                        line = ""
                        continue
                    if ord(char) == 8:
                        if not line_wait:
                            logger.info(line)
                            line = ""
                        line_wait = True
                        continue
                    if line_wait and -ord(char) == 32:
                        continue
                    line += char
                    line_wait = False

        elif command_type == "ffmpeg":
            last_write = 0
            for i, line in enumerate(self.process.stdout):
                if self.killed:
                    logger.info(line.rstrip())
                    break
                if not white_detect.match(line):
                    if "Skipping NAL unit" in line:
                        last_write -= 1
                        continue

                    line = line.strip()
                    if line.startswith("frame"):
                        if last_write + 50 < i:
                            last_write = i
                            logger.info(line.rstrip())
                    else:
                        logger.info(line.rstrip())
                    if line.startswith(("frame", "encoded")):
                        self.app.log_label_update(line.strip())

        return_code = self.process.poll()
        return return_code

    def run(self):
        try:
            for command in self.command_list:
                if self.killed:
                    self.tempdir.cleanup()
                    return
                if command.ensure_paths:
                    for path in command.ensure_paths:
                        path.mkdir(parents=True, exist_ok=True)
                if command.item == "command":
                    code = self.run_command(command.command, command.exe)
                    if code and not self.killed:
                        return self.app.completed.emit(str(code))
                elif command.item == "loop":
                    self.loop_creates(command.dirs, command.files)
                    for index, res in enumerate(command.condition(self.temp_files, self.temp_dirs)):
                        for item in command.commands:
                            cmd = self.replace_loop_args(item.command, index, res)
                            code = self.run_command(cmd, item.exe)
                            if code and not self.killed:
                                return self.app.completed.emit(str(code))
        except Exception as err:
            logger.exception(f"Could not run commands - {err}")
            self.tempdir.cleanup()
            if not self.killed:
                self.app.completed.emit(1)
        else:
            self.tempdir.cleanup()
            if not self.killed:
                self.app.completed.emit(0)

    def start_exec(self, command):
        return Popen(
            command,
            shell=True,
            cwd=self.tempdir.name,
            stdin=PIPE,
            stdout=PIPE,
            stderr=STDOUT,
            universal_newlines=True,
            preexec_fn=os.setsid if not reusables.win_based else None,
        )

    def is_alive(self):
        if not self.process:
            return False
        return True if self.process.poll() is None else False

    def kill(self):
        logger.info(f"Killing worker process {self.process.pid}")
        if self.process:
            try:
                if reusables.win_based:
                    os.kill(self.process.pid, signal.CTRL_C_EVENT)
                else:
                    os.killpg(os.getpgid(self.process.pid), signal.SIGTERM)
                self.process.terminate()
            except Exception as err:
                logger.exception(f"Couldn't terminate process: {err}")
        self.killed = True
        self.app.cancelled.emit()
        self.exit()



# -*- coding: utf-8 -*-
import reusables
import re

from box import Box

from fastflix.plugins.common.helpers import generate_filters, Command
from fastflix.plugins.common.audio import build_audio
from fastflix.plugins.common.subtitles import build_subtitle


def build(
    source,
    video_track,
    ffmpeg,
    bitrate=None,
    crf=None,
    start_time=0,
    duration=None,
    preset="fast",
    audio_tracks=(),
    subtitle_tracks=(),
    disable_hdr=False,
    side_data=None,
    x265_params=None,
    intra_encoding=False,
    max_mux="default",
    extra="",
    pix_fmt="yuv420p10le",
    tune=None,
    profile="default",
    **kwargs,
):
    filters = generate_filters(disable_hdr=disable_hdr, **kwargs)
    audio = build_audio(audio_tracks)
    subtitles = build_subtitle(subtitle_tracks)

    ending = "/dev/null"
    if reusables.win_based:
        ending = "NUL"

    if not side_data:
        side_data = Box(default_box=True)

    beginning = (
        f'"{ffmpeg}" -y '
        f'-i "{source}" '
        f' {f"-ss {start_time}" if start_time else ""}  '
        f'{f"-to {duration}" if duration else ""} '
        f"{extra} "
        f"-map 0:{video_track} "
        f"-pix_fmt {pix_fmt} "
        f"-c:v:0 libx265 "
        f'{f"-vf {filters}" if filters else ""} '
        f'{f"-tune {tune}" if tune else ""} '
        "-map_metadata -1 "
    )

    if profile and profile != "default":
        beginning += f"-profile {profile} "

    if max_mux and max_mux != "default":
        beginning += f"-max_muxing_queue_size {max_mux} "

    if not x265_params:
        x265_params = []

    if not disable_hdr:
        if side_data.color_primaries == "bt2020":
            x265_params.extend(
                ["hdr-opt=1", "repeat-headers=1", "colorprim=bt2020", "transfer=smpte2084", "colormatrix=bt2020nc"]
            )

        if side_data.master_display:
            x265_params.append(
                "master-display="
                f"G{side_data.master_display.green}"
                f"B{side_data.master_display.blue}"
                f"R{side_data.master_display.red}"
                f"WP{side_data.master_display.white}"
                f"L{side_data.master_display.luminance}"
            )

        if side_data.cll:
            x265_params.append(f"max-cll={side_data.cll}")

    if intra_encoding:
        x265_params.append("keyint=1")

    if side_data.cll:
        pass

    extra_data = "-map_chapters 0 "  # -map_metadata 0 # safe to do for rotation?

    def get_x265_params(params=()):
        if not isinstance(params, (list, tuple)):
            params = [params]
        all_params = x265_params + list(params)
        return '-x265-params "{}" '.format(":".join(all_params)) if all_params else ""

    if bitrate:
        command_1 = (
            f'{beginning} {get_x265_params(["pass=1"])} '
            f'-passlogfile "<tempfile.1.log>" -b:v {bitrate} -preset {preset} -an -sn -dn -f mp4 {ending}'
        )
        command_2 = (
            f'{beginning} {get_x265_params(["pass=2"])} -passlogfile "<tempfile.1.log>" '
            f'-b:v {bitrate} -preset {preset} {audio} {subtitles} {extra_data} "{{output}}"'
        )
        return [
            Command(
                re.sub("[ ]+", " ", command_1), ["ffmpeg", "output"], False, name="First pass bitrate", exe="ffmpeg"
            ),
            Command(
                re.sub("[ ]+", " ", command_2), ["ffmpeg", "output"], False, name="Second pass bitrate", exe="ffmpeg"
            ),
        ]

    elif crf:
        command = (
            f"{beginning} {get_x265_params()}  -crf {crf} "
            f'-preset {preset} {audio} {subtitles} {extra_data} "{{output}}"'
        )
        return [
            Command(re.sub("[ ]+", " ", command), ["ffmpeg", "output"], False, name="Single pass CRF", exe="ffmpeg")
        ]

    else:
        return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from box import Box

from qtpy import QtWidgets, QtCore, QtGui

from fastflix.shared import error_message, main_width
from fastflix.widgets.panels.abstract_list import FlixList

dispositions = [
    "none",
    "default",
    "dub",
    "original",
    "comment",
    "lyrics",
    "karaoke",
    "forced",
    "hearing_impaired",
]

languages = (
    "aar,abk,ace,ach,ada,ady,afh,afr,ain,aka,akk,ale,alt,amh,ang,anp,ara,arc,arg,arn,arp,arw,asm,ast,ava,ave,"
    "awa,aym,aze,bak,bal,bam,ban,bas,bej,bel,bem,ben,bho,bik,bin,bis,bla,bod,bos,bra,bre,bua,bug,bul,byn,cad,"
    "car,cat,ceb,ces,cha,chb,che,chg,chk,chm,chn,cho,chp,chr,chu,chv,chy,cop,cor,cos,cre,crh,csb,cym,dak,dan,"
    "dar,del,den,deu,dgr,din,div,doi,dsb,dua,dum,dyu,dzo,efi,egy,eka,ell,elx,eng,enm,epo,est,eus,ewe,ewo,fan,"
    "fao,fas,fat,fij,fil,fin,fon,fra,frm,fro,frr,frs,fry,ful,fur,gaa,gay,gba,gez,gil,gla,gle,glg,glv,gmh,goh,"
    "gon,gor,got,grb,grc,grn,gsw,guj,gwi,hai,hat,hau,haw,heb,her,hil,hin,hit,hmn,hmo,hrv,hsb,hun,hup,hye,iba,"
    "ibo,ido,iii,iku,ile,ilo,ina,ind,inh,ipk,isl,ita,jav,jbo,jpn,jpr,jrb,kaa,kab,kac,kal,kam,kan,kas,kat,kau,"
    "kaw,kaz,kbd,kha,khm,kho,kik,kin,kir,kmb,kok,kom,kon,kor,kos,kpe,krc,krl,kru,kua,kum,kur,kut,lad,lah,lam,"
    "lao,lat,lav,lez,lim,lin,lit,lol,loz,ltz,lua,lub,lug,lui,lun,luo,lus,mad,mag,mah,mai,mak,mal,man,mar,mas,"
    "mdf,mdr,men,mga,mic,min,mis,mkd,mlg,mlt,mnc,mni,moh,mon,mos,mri,msa,mul,mus,mwl,mwr,mya,myv,nap,nau,nav,"
    "nbl,nde,ndo,nds,nep,new,nia,niu,nld,nno,nob,nog,non,nor,nqo,nso,nwc,nya,nym,nyn,nyo,nzi,oci,oji,ori,orm,"
    "osa,oss,ota,pag,pal,pam,pan,pap,pau,peo,phn,pli,pol,pon,por,pro,pus,que,raj,rap,rar,roh,rom,ron,run,rup,"
    "rus,sad,sag,sah,sam,san,sas,sat,scn,sco,sel,sga,shn,sid,sin,slk,slv,sma,sme,smj,smn,smo,sms,sna,snd,snk,"
    "sog,som,sot,spa,sqi,srd,srn,srp,srr,ssw,suk,sun,sus,sux,swa,swe,syc,syr,tah,tam,tat,tel,tem,ter,tet,tgk,"
    "tgl,tha,tig,tir,tiv,tkl,tlh,tli,tmh,tog,ton,tpi,tsi,tsn,tso,tuk,tum,tur,tvl,twi,tyv,udm,uga,uig,ukr,umb,"
    "und,urd,uzb,vai,ven,vie,vol,vot,wal,war,was,wln,wol,xal,xho,yao,yap,yid,yor,zap,zbl,zen,zgh,zha,zho,zul,"
    "zun,zxx,zza"
)

supported_attachments = {
    "SRT Subtitles": {
        "ext": ["srt"],

    },
    "Thumbnail": {

    }
}

language_list = languages.split(",")


class Attachment(QtWidgets.QTabWidget):
    def __init__(self, parent, attachment, index, enabled=True, first=False):
        self.loading = True
        super(Attachment, self).__init__(parent)
        self.parent = parent
        self.index = index
        self.outdex = None
        self.attachment = Box(attachment, default_box=True)
        self.first = first
        self.last = False
        self.attachment_lang = attachment.get("tags", {}).get("language")

        self.widgets = Box(
            track_number=QtWidgets.QLabel(f"{self.index}:{self.outdex}" if enabled else "❌"),
            title=QtWidgets.QLabel(f"  {self.attachment.codec_long_name}"),
            up_button=QtWidgets.QPushButton("^"),
            down_button=QtWidgets.QPushButton("v"),
            enable_check=QtWidgets.QCheckBox("Preserve"),
            disposition=QtWidgets.QComboBox(),
            language=QtWidgets.QComboBox(),
        )

        self.widgets.disposition.addItems(dispositions)
        self.widgets.enable_check.setChecked(enabled)
        self.widgets.enable_check.toggled.connect(self.update_enable)
        self.widgets.disposition.currentIndexChanged.connect(self.page_update)
        self.widgets.disposition.setCurrentIndex(0)
        for disposition, is_set in self.attachment.disposition.items():
            if is_set:
                try:
                    self.widgets.disposition.setCurrentIndex(dispositions.index(disposition))
                except ValueError:
                    pass  # TODO figure out all possible dispositions for attachments / log if issue
                break

        self.setFixedHeight(60)

        disposition_layout = QtWidgets.QHBoxLayout()
        disposition_layout.addStretch()
        disposition_layout.addWidget(QtWidgets.QLabel("Disposition"))
        disposition_layout.addWidget(self.widgets.disposition)

        grid = QtWidgets.QGridLayout()
        grid.addLayout(self.init_move_buttons(), 0, 0)
        grid.addWidget(self.widgets.track_number, 0, 1)
        grid.addWidget(self.widgets.title, 0, 2)
        grid.addLayout(disposition_layout, 0, 4)
        grid.addLayout(self.init_language(), 0, 5)
        grid.addWidget(self.widgets.enable_check, 0, 6)

        self.setLayout(grid)
        self.loading = False

    def init_move_buttons(self):
        layout = QtWidgets.QVBoxLayout()
        layout.setSpacing(0)
        # layout.setMargin(0)
        # self.widgets.up_button = QtWidgets.QPushButton("^")
        self.widgets.up_button.setDisabled(self.first)
        self.widgets.up_button.setFixedWidth(20)
        self.widgets.up_button.clicked.connect(lambda: self.parent.move_up(self))
        # self.widgets.down_button = QtWidgets.QPushButton("v")
        self.widgets.down_button.setDisabled(self.last)
        self.widgets.down_button.setFixedWidth(20)
        self.widgets.down_button.clicked.connect(lambda: self.parent.move_down(self))
        layout.addWidget(self.widgets.up_button)
        layout.addWidget(self.widgets.down_button)
        return layout

    def init_language(self):
        self.widgets.language.addItems(language_list)
        try:
            self.widgets.language.setCurrentIndex(language_list.index(self.attachment_lang))
        except Exception:
            self.widgets.language.setCurrentIndex(language_list.index("eng"))

        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(QtWidgets.QLabel("Language"))
        layout.addWidget(self.widgets.language)
        return layout

    def set_first(self, first=True):
        self.first = first
        self.widgets.up_button.setDisabled(self.first)

    def set_last(self, last=True):
        self.last = last
        self.widgets.down_button.setDisabled(self.last)

    def set_outdex(self, outdex):
        self.outdex = outdex
        if not self.enabled:
            self.widgets.track_number.setText("❌")
        else:
            self.widgets.track_number.setText(f"{self.index}:{self.outdex}")

    @property
    def disposition(self):
        text = self.widgets.disposition.currentText()
        return 0 if text == "none" else text

    @property
    def enabled(self):
        return self.widgets.enable_check.isChecked()

    @property
    def language(self):
        return self.widgets.language.currentText()

    def update_enable(self):
        enabled = self.widgets.enable_check.isChecked()
        self.widgets.track_number.setText(f"{self.index}:{self.outdex}" if enabled else "❌")
        self.parent.reorder()
        self.page_update()

    def page_update(self):
        if not self.loading:
            return self.parent.main.page_update()


class AttachmentList(FlixList):
    def __init__(self, parent, starting_pos=0):
        super().__init__(parent, "Attachments", starting_pos)
        self.main = parent.main

    def new_source(self, starting_pos=0):
        self.starting_pos = starting_pos
        self.tracks = []
        for index, track in enumerate(self.main.streams.attachment):
            new_item = Attachment(self, track, index=track.index, first=True if index == 0 else False)
            self.tracks.append(new_item)
        if self.tracks:
            self.tracks[0].set_first()
            self.tracks[-1].set_last()

        super()._new_source(self.tracks)

    def get_settings(self):
        tracks = []
        for track in self.tracks:
            if track.enabled:
                tracks.append(
                    {
                        "index": track.index,
                        "outdex": track.outdex,
                        "disposition": track.disposition,
                        "language": track.language,
                    }
                )
        return Box(attachment_tracks=tracks)



# -*- coding: utf-8 -*-
from fastflix.plugins.common.helpers import generate_filters, Command, start_and_input

extension = "gif"


def build(source, video_track, ffmpeg, fps=15, dither="sierra2_4a", **kwargs):

    filters = generate_filters(**kwargs)

    beginning = start_and_input(source, ffmpeg, **kwargs)

    command_1 = (
        f"{beginning} -map 0:{video_track} "
        f'-vf "{f"{filters}," if filters else "" }palettegen" -y "<tempfile.1.png>"'
    )

    gif_filters = f"fps={fps:.2f}"
    if filters:
        gif_filters += f",{filters}"

    command_2 = (
        f'{beginning} -i "<tempfile.1.png>" -map 0:{video_track} '
        f'-lavfi "{gif_filters} [x]; [x][1:v] paletteuse=dither={dither}" -y "{{output}}" '
    )

    return [
        Command(command_1, ["ffmpeg", "pallet_file", "output"], False, name="Pallet generation", exe="ffmpeg"),
        Command(command_2, ["ffmpeg", "pallet_file", "output"], False, name="GIF creation", exe="ffmpeg"),
    ]



# -*- coding: utf-8 -*-
import logging

from box import Box

from qtpy import QtWidgets, QtCore, QtGui

logger = logging.getLogger("fastflix")

recommended_bitrates = [
    "150k   (320x240p @ 24,25,30)",
    "276k   (640x360p @ 24,25,30)",
    "512k   (640x480p @ 24,25,30)",
    "1024k  (1280x720p @ 24,25,30)",
    "1800k (1280x720p @ 50,60)",
    "1800k (1920x1080p @ 24,25,30)",
    "3000k (1920x1080p @ 50,60)",
    "6000k (2560x1440p @ 24,25,30)",
    "9000k (2560x1440p @ 50,60)",
    "12000k (3840x2160p @ 24,25,30)",
    "18000k (3840x2160p @ 50,60)",
    "Custom",
]

recommended_crfs = [
    "37 (240p)",
    "36 (360p)",
    "33 (480p)",
    "32 (720p)",
    "31 (1080p)",
    "24 (1440p)",
    "15 (2160p)",
    "Custom",
]


class VP9(QtWidgets.QWidget):
    def __init__(self, parent, main):
        super(VP9, self).__init__(parent)
        self.main = main

        grid = QtWidgets.QGridLayout()

        # grid.addWidget(QtWidgets.QLabel("VP9"), 0, 0)

        self.widgets = Box(fps=None, remove_hdr=None, mode=None)

        self.mode = "CRF"

        grid.addLayout(self.init_remove_hdr(), 2, 0, 1, 2)
        grid.addLayout(self.init_modes(), 0, 2, 4, 4)
        grid.addLayout(self.init_quality(), 1, 0, 1, 2)
        grid.addLayout(self.init_speed(), 0, 0, 1, 2)

        grid.addLayout(self.init_row_mt(), 4, 0, 1, 2)
        grid.addLayout(self.init_force_420(), 5, 0, 1, 2)
        grid.addLayout(self.init_single_pass(), 6, 0, 1, 2)

        grid.addWidget(QtWidgets.QWidget(), 8, 0)
        grid.setRowStretch(8, 1)
        guide_label = QtWidgets.QLabel(
            f"<a href='https://trac.ffmpeg.org/wiki/Encode/VP9'>FFMPEG VP9 Encoding Guide</a>"
        )
        guide_label.setAlignment(QtCore.Qt.AlignBottom)
        guide_label.setOpenExternalLinks(True)
        grid.addWidget(guide_label, 9, 0, -1, 1)
        self.setLayout(grid)
        self.hide()

    # def init_fps(self):
    #     layout = QtWidgets.QHBoxLayout()
    #     layout.addWidget(QtWidgets.QLabel("FPS"))
    #     self.widgets.fps = QtWidgets.QComboBox()
    #     self.widgets.fps.addItems([str(x) for x in range(1, 31)])
    #     self.widgets.fps.setCurrentIndex(14)
    #     self.widgets.fps.currentIndexChanged.connect(lambda: self.main.build_commands())
    #     layout.addWidget(self.widgets.fps)
    #     return layout

    def init_remove_hdr(self):
        layout = QtWidgets.QHBoxLayout()
        remove_hdr_level = QtWidgets.QLabel("Remove HDR")
        remove_hdr_level.setToolTip(
            "Convert BT2020 colorspace into bt709\n " "WARNING: This will take much longer and result in a larger file"
        )
        layout.addWidget(remove_hdr_level)
        self.widgets.remove_hdr = QtWidgets.QComboBox()
        self.widgets.remove_hdr.addItems(["No", "Yes"])
        self.widgets.remove_hdr.setCurrentIndex(0)
        self.widgets.remove_hdr.setDisabled(True)
        self.widgets.remove_hdr.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.remove_hdr)
        return layout

    def init_quality(self):
        layout = QtWidgets.QHBoxLayout()
        quality_level = QtWidgets.QLabel("Quality")
        quality_level.setToolTip(
            "good is the default and recommended for most applications <br> "
            "best is recommended if you have lots of time and want the best compression efficiency."
        )
        layout.addWidget(quality_level)
        self.widgets.quality = QtWidgets.QComboBox()
        self.widgets.quality.addItems(["realtime", "good", "best"])
        self.widgets.quality.setCurrentIndex(1)
        self.widgets.quality.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.quality)
        return layout

    def init_speed(self):
        layout = QtWidgets.QHBoxLayout()
        speed_level = QtWidgets.QLabel("Speed")
        speed_level.setToolTip(
            "Using 1 or 2 will increase encoding speed at the expense of having some impact on "
            "quality and rate control accuracy.<br> 4 or 5 will turn off rate distortion optimization, "
            "having even more of an impact on quality."
        )
        layout.addWidget(speed_level)
        self.widgets.speed = QtWidgets.QComboBox()
        self.widgets.speed.addItems([str(x) for x in range(6)])
        self.widgets.speed.setCurrentIndex(0)
        self.widgets.speed.currentIndexChanged.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.speed)
        return layout

    def init_row_mt(self):
        layout = QtWidgets.QHBoxLayout()
        row_mt_label = QtWidgets.QLabel("Row multithreading")
        row_mt_label.setToolTip(
            "This improves encoding speed significantly on systems that "
            "are otherwise underutilised when encoding VP9."
        )
        layout.addWidget(row_mt_label)
        self.widgets.row_mt = QtWidgets.QCheckBox()
        self.widgets.row_mt.setChecked(False)
        self.widgets.row_mt.toggled.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.row_mt)
        return layout

    def init_force_420(self):
        layout = QtWidgets.QHBoxLayout()
        sub_label = QtWidgets.QLabel("Force 4:2:0 chroma subsampling")
        sub_label.setToolTip(
            "If your input source is using 4:2:2 or 4:4:4 chroma subsampling, "
            "some players might not be able to handle the output that libvpx produces."
        )
        layout.addWidget(sub_label)
        self.widgets.force_420 = QtWidgets.QCheckBox()
        self.widgets.force_420.setChecked(True)
        self.widgets.force_420.toggled.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.force_420)
        return layout

    def init_single_pass(self):
        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(QtWidgets.QLabel("Single Pass (CRF)"))
        self.widgets.single_pass = QtWidgets.QCheckBox()
        self.widgets.single_pass.setChecked(False)
        self.widgets.single_pass.toggled.connect(lambda: self.main.page_update())
        layout.addWidget(self.widgets.single_pass)
        return layout

    def init_modes(self):
        layout = QtWidgets.QGridLayout()
        crf_group_box = QtWidgets.QGroupBox()
        crf_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        crf_box_layout = QtWidgets.QHBoxLayout()
        bitrate_group_box = QtWidgets.QGroupBox()
        bitrate_group_box.setStyleSheet("QGroupBox{padding-top:5px; margin-top:-18px}")
        bitrate_box_layout = QtWidgets.QHBoxLayout()
        # rotation_dir = Path(base_path, 'data', 'rotations')
        # group_box.setStyleSheet("QGroupBox{padding-top:15px; margin-top:-15px; padding-bottom:-5px}")
        self.widgets.mode = QtWidgets.QButtonGroup()
        self.widgets.mode.buttonClicked.connect(self.set_mode)

        bitrate_radio = QtWidgets.QRadioButton("Bitrate")
        bitrate_radio.setFixedWidth(80)
        self.widgets.mode.addButton(bitrate_radio)
        self.widgets.bitrate = QtWidgets.QComboBox()
        self.widgets.bitrate.setFixedWidth(250)
        self.widgets.bitrate.addItems(recommended_bitrates)
        self.widgets.bitrate.setCurrentIndex(6)
        self.widgets.bitrate.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_bitrate = QtWidgets.QLineEdit("3000")
        self.widgets.custom_bitrate.setFixedWidth(100)
        self.widgets.custom_bitrate.setDisabled(True)
        self.widgets.custom_bitrate.textChanged.connect(lambda: self.main.build_commands())
        bitrate_box_layout.addWidget(bitrate_radio)
        bitrate_box_layout.addWidget(self.widgets.bitrate)
        bitrate_box_layout.addStretch()
        bitrate_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        bitrate_box_layout.addWidget(self.widgets.custom_bitrate)

        crf_radio = QtWidgets.QRadioButton("CRF")
        crf_radio.setChecked(True)
        crf_radio.setFixedWidth(80)
        self.widgets.mode.addButton(crf_radio)

        self.widgets.crf = QtWidgets.QComboBox()
        self.widgets.crf.setFixedWidth(250)
        self.widgets.crf.addItems(recommended_crfs)
        self.widgets.crf.setCurrentIndex(4)
        self.widgets.crf.currentIndexChanged.connect(lambda: self.mode_update())
        self.widgets.custom_crf = QtWidgets.QLineEdit("30")
        self.widgets.custom_crf.setFixedWidth(100)
        self.widgets.custom_crf.setDisabled(True)
        self.widgets.custom_crf.textChanged.connect(lambda: self.main.build_commands())
        crf_box_layout.addWidget(crf_radio)
        crf_box_layout.addWidget(self.widgets.crf)
        crf_box_layout.addStretch()
        crf_box_layout.addWidget(QtWidgets.QLabel("Custom:"))
        crf_box_layout.addWidget(self.widgets.custom_crf)

        bitrate_group_box.setLayout(bitrate_box_layout)
        crf_group_box.setLayout(crf_box_layout)

        layout.addWidget(crf_group_box, 0, 0)
        layout.addWidget(bitrate_group_box, 1, 0)
        return layout

    def mode_update(self):
        self.widgets.custom_crf.setDisabled(self.widgets.crf.currentText() != "Custom")
        self.widgets.custom_bitrate.setDisabled(self.widgets.bitrate.currentText() != "Custom")
        self.main.build_commands()

    def get_settings(self):
        settings = Box(
            disable_hdr=bool(self.widgets.remove_hdr.currentIndex()),
            quality=self.widgets.quality.currentText(),
            speed=self.widgets.speed.currentText(),
            row_mt=int(self.widgets.row_mt.isChecked()),
            force_420=self.widgets.force_420.isChecked(),
            single_pass=self.widgets.single_pass.isChecked(),
        )
        if self.mode == "CRF":
            crf = self.widgets.crf.currentText()
            settings.crf = int(crf.split(" ", 1)[0]) if crf.lower() != "custom" else self.widgets.custom_crf.text()
        else:
            bitrate = self.widgets.bitrate.currentText()
            if bitrate.lower() == "custom":
                settings.bitrate = self.widgets.custom_bitrate.currentText()
            else:
                settings.bitrate = bitrate.split(" ", 1)[0]
        return settings

    def new_source(self):
        if not self.main.streams:
            return
        if "zcale" not in self.main.flix.filters:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#777}")
            self.remove_hdr_label.setToolTip("cannot remove HDR, zcale filter not in current version of FFmpeg")
            logger.warning("zcale filter not detected in current version of FFmpeg, cannot remove HDR")
        elif self.main.streams["video"][self.main.video_track].get("color_space", "").startswith("bt2020"):
            self.widgets.remove_hdr.setDisabled(False)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")
        else:
            self.widgets.remove_hdr.setDisabled(True)
            self.remove_hdr_label.setStyleSheet("QLabel{color:#000}")

    def set_mode(self, x):
        self.mode = x.text()
        self.main.build_commands()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastflix',
 'fastflix.builders',
 'fastflix.plugins',
 'fastflix.plugins.av1_aom',
 'fastflix.plugins.common',
 'fastflix.plugins.gif',
 'fastflix.plugins.hevc_x265',
 'fastflix.plugins.svt_av1',
 'fastflix.plugins.vp9',
 'fastflix.widgets',
 'fastflix.widgets.panels']

package_data = \
{'': ['*'], 'fastflix': ['data/*', 'data/rotations/*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0',
 'pyside2>=5.15.0,<6.0.0',
 'python-box[all]>=5.1.1,<6.0.0',
 'qtpy>=1.9.0,<2.0.0',
 'requests>=2.24.0,<3.0.0',
 'reusables>=0.9.5,<0.10.0',
 'ruamel.yaml>=0.16.10,<0.17.0']

entry_points = \
{'console_scripts': ['fastflix = fastflix.__main__:main']}

setup_kwargs = {
    'name': 'fastflix',
    'version': '2.6.0',
    'description': 'Easy to use video encoder GUI wrapper',
    'long_description': '[![Build status](https://ci.appveyor.com/api/projects/status/208k29cvoq8xwf8j/branch/master?svg=true)](https://ci.appveyor.com/project/cdgriffith/fastflix/branch/master)\n\n# FastFlix\n\nFastFlix is a AV1, HEVC (x265) and VP9 encoder, GIF maker, and general ffmpeg command wrapper.\n\n![preview](https://raw.githubusercontent.com/cdgriffith/binary-files/fast-flix/media/fastflix/2.0.0/main.png)\n\n\n# Encoders\n\nCurrently there is support for:\n\n* HEVC (libx265)\n* AV1 (SVT-AV1 on Windows)\n* AV1 (FFMPEG libaom - currently very slow)\n* VP9\n* GIF\n\n\n# Releases\n\n## Windows\nView the [releases](https://github.com/cdgriffith/FastFlix/releases) for 64 bit Windows binaries (Generated via Appveyor and also [available there](https://ci.appveyor.com/project/cdgriffith/fastflix)).\n\n## MacOS and Linux\n\nPlease use [pipx](https://pipxproject.github.io/pipx/installation/) to install as a properly virtualized app\n\n```\npipx install fastflix\n```\n\n## Running from source code\n\n```\ngit clone https://github.com/cdgriffith/FastFlix.git\ncd FastFlix\npython3 -m venv venv\n. venv/bin/activate\npip install -r requirements.txt\npython -m flix\n```\n\n# License\n\nCopyright (C) 2019-2020 Chris Griffith\n\nThe code itself is licensed under the MIT which you can read in the `LICENSE` file.\nRead more about the release licensing in the [docs](docs/README.md) folder.\n\n',
    'author': 'Chris Griffith',
    'author_email': 'chris@cdgriffith.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
import sys
import logging
from logging.handlers import SocketHandler
from pathlib import Path
from distutils.version import StrictVersion
from datetime import datetime
import os
import shutil
import traceback
from json import JSONDecodeError
import coloredlogs

try:
    import pkg_resources.py2_warn  # Needed for pyinstaller on 3.8
except ImportError:
    pass

try:
    from appdirs import user_data_dir
    from box import Box
    import reusables
    import requests

    from qtpy import QtWidgets, QtCore, QtGui
    from qtpy import QT_VERSION, API

    from fastflix.version import __version__
    from fastflix.flix import ff_version, FlixError
    from fastflix.shared import error_message, base_path, message
    from fastflix.widgets.container import Container
except ImportError as err:
    traceback.print_exc()
    print("Could not load FastFlix properly!", file=sys.stderr)
    input("Please report this issue on https://github.com/cdgriffith/FastFlix/issues (press any key to exit)")
    sys.exit(1)

logger = logging.getLogger("fastflix")


def main():
    logging.basicConfig(level=logging.DEBUG, format="%(asctime)s - %(name)-6s  %(levelname)-8s %(message)s")
    socket_handler = SocketHandler("127.0.0.1", 19996)
    logger.addHandler(socket_handler)
    coloredlogs.install(level="DEBUG", logger=logger)

    logger.info(f"Starting FastFlix {__version__}")
    logger.debug(f"Using qt engine {API} version {QT_VERSION}")

    main_app = QtWidgets.QApplication(sys.argv)
    main_app.setStyle("fusion")
    main_app.setApplicationDisplayName("FastFlix")

    data_path = Path(user_data_dir("FastFlix", appauthor=False, roaming=True))
    ffmpeg_folder = Path(user_data_dir("FFmpeg", appauthor=False, roaming=True))
    data_path.mkdir(parents=True, exist_ok=True)
    log_dir = data_path / "logs"
    log_dir.mkdir(parents=True, exist_ok=True)

    ffmpeg = shutil.which("ffmpeg")
    if ffmpeg:
        ffmpeg = Path(ffmpeg)
    ffprobe = shutil.which("ffprobe")
    if ffprobe:
        ffprobe = Path(ffprobe)
    svt_av1 = shutil.which("SvtAv1EncApp")

    if ffmpeg_folder.exists():
        for file in ffmpeg_folder.iterdir():
            if file.is_file() and file.name.lower() in ("ffmpeg", "ffmpeg.exe"):
                ffmpeg = file
            if file.is_file() and file.name.lower() in ("ffprobe", "ffprobe.exe"):
                ffprobe = file
        if (not ffmpeg or not ffprobe) and (ffmpeg_folder / "bin").exists():
            for file in (ffmpeg_folder / "bin").iterdir():
                if file.is_file() and file.name.lower() in ("ffmpeg", "ffmpeg.exe"):
                    ffmpeg = file
                if file.is_file() and file.name.lower() in ("ffprobe", "ffprobe.exe"):
                    ffprobe = file

    logger.addHandler(logging.FileHandler(log_dir / f"flix_{datetime.now().isoformat().replace(':', '.')}"))

    config_file = Path(data_path, "fastflix.json")
    if not config_file.exists():
        config = Box({"version": __version__, "work_dir": str(data_path)})
        config.to_json(filename=config_file, indent=2)
    else:
        try:
            config = Box.from_json(filename=config_file)
        except JSONDecodeError as err:
            logger.exception(f'Error with config file: "{config_file}"')
            error_message(
                msg=f"Bad config file: {config_file}"
                "<br> If you are unsure what to do, just delete the file"
                f"<br><br>Error: {err}",
                traceback=True,
            )
            sys.exit(1)
        if "version" not in config or "work_dir" not in config:
            message("Config file does not have all required fields, adding defaults")
            config.version = __version__
            config.work_dir = str(data_path)
            config.to_json(filename=config_file, indent=2)
        if StrictVersion(config.version) < StrictVersion(__version__):
            # do upgrade of config
            config.version = __version__
            config.to_json(filename=config_file, indent=2)
        if "ffmpeg" in config:
            ffmpeg = Path(config.ffmpeg)
        if "ffprobe" in config:
            ffprobe = Path(config.ffprobe)
        if "svt_av1" in config:
            svt_av1 = Path(config.svt_av1)
    work_dir = Path(config.get("work_dir", data_path))
    if not work_dir.exists():
        try:
            work_dir.mkdir(parents=True, exist_ok=True)
        except OSError as err:
            logger.error(
                f"Cannot use specified working directory {work_dir}"
                f" - Falling back to {data_path} due to error: {err}"
            )
            work_dir = data_path
            work_dir.mkdir(parents=True, exist_ok=True)

    if not ffmpeg or not ffprobe:
        qm = QtWidgets.QMessageBox
        if reusables.win_based:
            ret = qm.question(
                None,
                "FFmpeg not found!",
                f"<h2>FFmpeg not found!</h2>" f"<br> Automatically download FFmpeg?",
                qm.Yes | qm.No,
            )
            if ret == qm.Yes:
                try:
                    windows_download_ffmpeg(ffmpeg_folder)
                except Exception as err:
                    logger.exception("Could not download FFmpeg")
                    sys.exit(2)
                else:
                    ffmpeg = ffmpeg_folder / "bin" / "ffmpeg.exe"
                    ffprobe = ffmpeg_folder / "bin" / "ffprobe.exe"
            else:
                sys.exit(1)
        else:
            qm.question(
                None, "<h2>FFmpeg not found!</h2>", "Please download FFmpeg via your platform package manager", qm.Close
            )
            sys.exit(1)
    else:
        logger.info(f"Using ffmpeg {ffmpeg}")
        logger.info(f"Using ffprobe {ffprobe}")

    svt_av1_folder = Path(user_data_dir("SVT-AV1", appauthor=False, roaming=True))
    if not svt_av1 and svt_av1_folder.exists():
        svt_av1 = Path(svt_av1_folder, "SvtAv1EncApp.exe")

    if (not svt_av1 or not svt_av1.exists()) and reusables.win_based:
        qm = QtWidgets.QMessageBox
        ret = qm.question(
            None,
            "Download SVT-AV1",
            f"<h2>Would you like to download SVT-AV1?</h2>" f"<br> Will be placed in:" f"<br> {svt_av1_folder}",
            qm.Yes | qm.No,
        )
        if ret == qm.Yes:
            svt_av1_folder.mkdir(parents=True, exist_ok=True)
            try:
                download_svt_av1(svt_av1_folder)
            except Exception:
                logging.exception("Could not download newest SVT-AV1!")
                qm.question(None, "", f"Could not download SVT-AV1!", qm.Close)
                sys.exit(1)
            else:
                svt_av1 = Path(svt_av1_folder, "SvtAv1EncApp.exe")
    try:
        ffmpeg_version = ff_version(ffmpeg, throw=True)
        ffprobe_version = ff_version(ffprobe, throw=True)
    except FlixError:
        error_message("ffmpeg or ffmpeg could not be executed properly!")
        sys.exit(1)

    try:
        window = Container(
            ffmpeg=ffmpeg,
            ffprobe=ffprobe,
            ffmpeg_version=ffmpeg_version,
            ffprobe_version=ffprobe_version,
            svt_av1=svt_av1,
            source=sys.argv[1] if len(sys.argv) > 1 else "",
            data_path=data_path,
            work_path=work_dir,
            config_file=config_file,
        )
        window.show()
    except (Exception, BaseException, SystemError, SystemExit):
        logger.exception("HARD FAIL: Unexpected error")
        sys.exit(1)
    sys.exit(main_app.exec_())


def download_svt_av1(svt_av1_folder):
    logger.info(f"Downloading SVT-AV1 to {svt_av1_folder}")

    if reusables.win_based:
        svt_av1_releases = requests.get(f"https://api.github.com/repos/OpenVisualCloud/SVT-AV1/releases").json()
        svt_av1_assets = requests.get(svt_av1_releases[0]["assets_url"]).json()
        for asset in svt_av1_assets:
            reusables.download(asset["browser_download_url"], save_dir=svt_av1_folder)


def windows_download_ffmpeg(ffmpeg_folder):
    ffmpeg_folder.mkdir(exist_ok=True)
    url = "https://ffmpeg.zeranoe.com/builds/win64/static/ffmpeg-latest-win64-static.zip"
    logger.info(f"Downloading {url} to {ffmpeg_folder}")
    req = requests.get(url, headers={"referer": "https://ffmpeg.zeranoe.com/"}, stream=True)
    with open(ffmpeg_folder / "ffmpeg-latest-win64-static.zip", "wb") as f:
        for block in req.iter_content(chunk_size=4096):
            f.write(block)

    reusables.extract(ffmpeg_folder / "ffmpeg-latest-win64-static.zip", path=ffmpeg_folder)
    sub_dir = ffmpeg_folder / "ffmpeg-latest-win64-static"

    for item in os.listdir(sub_dir):
        shutil.move(str(sub_dir / item), str(ffmpeg_folder))

    try:
        sub_dir.unlink()
        Path("ffmpeg-latest-win64-static.zip").unlink()
    except OSError:
        pass


if __name__ == "__main__":
    try:
        main()
    except Exception:
        traceback.print_exc()
        input(
            "Error while running FastFlix!\n"
            "Plese report this issue on https://github.com/cdgriffith/FastFlix/issues (press any key to exit)"
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Chris Griffith"

name = "GIF"

video_extension = "gif"
video_dimension_divisor = 1

enable_subtitles = False
enable_audio = False
enable_attachments = False

audio_formats = []

from fastflix.plugins.gif.command_builder import build
from fastflix.plugins.gif.settings_panel import GIF as settings_panel



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import math

from box import Box

from qtpy import QtWidgets, QtCore, QtGui


class Loop(QtWidgets.QGroupBox):
    def __init__(self, parent, condition, commands, number, name=""):
        super(Loop, self).__init__(parent)
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(QtWidgets.QLabel(f"Loop: {name}"))
        self.condition = condition
        self.number = number
        self.setStyleSheet("QGroupBox{padding-top:15px; margin-top:-18px}")

        for index, item in enumerate(commands, 1):
            new_item = Command(parent, item.command, index, item.name)
            layout.addWidget(new_item)
        self.setLayout(layout)


class Command(QtWidgets.QTabWidget):
    def __init__(self, parent, command, number, name="", enabled=True):
        super(Command, self).__init__(parent)
        self.command = command
        self.widget = QtWidgets.QTextEdit()
        self.widget.setReadOnly(True)
        self.widget.setText(command)
        self.widget.setDisabled(not enabled)
        self.setMinimumHeight(45 + (math.ceil(len(command) / 180) * 25))

        grid = QtWidgets.QGridLayout()
        grid.addWidget(QtWidgets.QLabel(f"Command {number}" if not name else name), 0, 0, 1, 2)
        grid.addWidget(self.widget, 1, 0, 1, 2)
        self.setLayout(grid)


class CommandList(QtWidgets.QWidget):
    def __init__(self, parent):
        super(CommandList, self).__init__(parent)

        layout = QtWidgets.QGridLayout()
        layout.addWidget(QtWidgets.QLabel("Commands to execute"))

        self.inner_widget = QtWidgets.QWidget()

        self.scroll_area = QtWidgets.QScrollArea(self)
        self.scroll_area.setMinimumHeight(200)

        layout.addWidget(self.scroll_area)

        self.setLayout(layout)

    def update_commands(self, commands):
        if not commands:
            return
        self.inner_widget = QtWidgets.QWidget()
        sp = QtWidgets.QSizePolicy()
        sp.setHorizontalPolicy(QtWidgets.QSizePolicy.Policy.Maximum)
        self.inner_widget.setSizePolicy(sp)
        layout = QtWidgets.QVBoxLayout()
        layout.setSpacing(5)
        for index, item in enumerate(commands, 1):
            if item.item == "command":
                new_item = Command(self.scroll_area, item.command, index, name=item.name)
                layout.addWidget(new_item)
            elif item.item == "loop":
                new_item = Loop(self.scroll_area, item.condition, item.commands, index, name=item.name)
                layout.addWidget(new_item)
        layout.addStretch()
        self.inner_widget.setLayout(layout)
        self.scroll_area.setWidget(self.inner_widget)
        self.scroll_area.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.inner_widget.setFixedWidth(self.scroll_area.width() - 3)

    def resizeEvent(self, event: QtGui.QResizeEvent):
        self.inner_widget.setFixedWidth(self.scroll_area.width() - 3)
        return super(CommandList, self).resizeEvent(event)
